@@ -16,6 +16,7 @@ public class ClassType extends ObjectType
   public static final int JDK_1_6_VERSION = 50 * 0x10000 + 0;
   public static final int JDK_1_7_VERSION = 51 * 0x10000 + 0;
   public static final int JDK_1_8_VERSION = 52 * 0x10000 + 0;
+  public static final int JAVA_9_VERSION = 53 * 0x10000 + 0;
 
   // An old but generally valid default value.
   int classfileFormatVersion = JDK_1_1_VERSION;
@@ -882,7 +883,7 @@ public class ClassType extends ObjectType
     int count = 0;
     String inheritingPackage = null;
     for (ClassType ctype = this;  ctype != null;
-	 ctype = ctype.getSuperclass())
+	 ctype = ctype.isInterface() ? Type.objectType : ctype.getSuperclass())
     {
       String curPackage = ctype.getPackageName();
       for (Method meth = ctype.getDeclaredMethods();
@@ -901,11 +902,7 @@ public class ClassType extends ObjectType
             {
               if (result != null)
                 {
-                  /* #ifdef JAVA2 */
                   result.add(meth);
-                  /* #else */
-                  // result.addElement(meth);
-                  /* #endif */
                 }
               count++;
             }
@@ -915,17 +912,14 @@ public class ClassType extends ObjectType
 
       if (searchSupers == 0)
 	break;
+    }
 
-      if (searchSupers > 1)
-	{
-	  ClassType[] interfaces = ctype.getAllInterfaces();
-	  if (interfaces != null)
-	    {
-	      for (int i = 0;  i < interfaces.length;  i++)
-		count += interfaces[i].getMethods(filter, searchSupers,
-						  result);
-	    }
-	}
+    if (searchSupers > 1) {
+        ClassType[] interfaces = getAllInterfaces();
+        if (interfaces != null) {
+            for (int i = 0;  i < interfaces.length;  i++)
+		count += interfaces[i].getMethods(filter, 0, result);
+        }
     }
     return count;
   }
@@ -1406,12 +1400,7 @@ public class ClassType extends ObjectType
         }
         int comp = compare(valueType);
         if (comp >= 0) {
-            if (valueType instanceof ClassType
-                || valueType instanceof ParameterizedType
-                || valueType instanceof TypeVariable)
-                return 2;
-            else
-                return 1;
+            return valueType instanceof ObjectType ? 2 : 1;
         } else
             return comp == -3 ? -1 : 0;
     }
@@ -1504,12 +1493,17 @@ public class ClassType extends ObjectType
     for (int i = 0;  i < nmethods;  i++)
       {
         Method meth = methods[i];
-        String mname = meth.getName();
-        Type[] ptypes = meth.getParameterTypes();
-
-        Method mimpl = getMethod(mname, ptypes);
-        if (mimpl != null && ! mimpl.isAbstract())
-          continue;
+        final String mname = meth.getName();
+        final String sig = meth.getSignature();
+        Filter<Method> filter = new Filter<Method>() {
+                 public boolean select(Method m) {
+                     return (! m.isAbstract()
+                             && mname.equals(m.getName())
+                             && sig.equals(m.getSignature()));
+                 }
+            };
+        if (countMethods(filter, 2) > 0)
+            continue;
         if (result != null)
           return null;
         result = meth;
