@@ -35,7 +35,8 @@ public class CodeAttr extends Attribute implements AttrContainer
   int flags;
 
   public Type[] stack_types;
-  Type[] local_types;
+  Type[] local_types = new Type[20];
+
   /** Previously-defined label. */
   Label previousLabel;
   /** Set of vars set in current block (since previousLabel).
@@ -65,17 +66,16 @@ public class CodeAttr extends Attribute implements AttrContainer
      in exception_table. */
   int exception_table_length;
 
-  /** Not a fixup - a no-op. */
-  static final int FIXUP_NONE = 0;
   /** The definition of a label. */
-  static final int FIXUP_DEFINE = 1;
+  static final int FIXUP_DEFINE = 0;
+  static final int FIXUP_DEFINE_UNREACHABLE = 1;
+  // From FIXUP_SWITCH up to FIXUP_TRANSFER2 must be contiguous
+  // - see the jump-to-jump optimization in processFixups.
   /** The offset points to a tableswitch/lookupswitch - handle padding. */
   static final int FIXUP_SWITCH = 2;
   /** The offset contains a label relative to the previous FIXUP_SWITCH. */
   static final int FIXUP_CASE = 3;
-  /** The offset points to a goto instruction.
-   * This case up to FIXUP_TRANSFER2 must be contiguous
-   * - see the jump-to-jump optimization in processFixups. */
+  /** The offset points to a goto instruction. */
   static final int FIXUP_GOTO = 4;
   /** The offset points to a jsr instruction. */
   static final int FIXUP_JSR = 5;
@@ -127,8 +127,9 @@ public class CodeAttr extends Attribute implements AttrContainer
    */
   public final void fixupChain (Label here, Label target)
   {
-    fixupAdd(CodeAttr.FIXUP_MOVE, 0, target);
+    fixupAdd(CodeAttr.FIXUP_MOVE, -1, target);
     here.defineRaw(this);
+    setPreviousLabelHere(here);
   }
 
   /** Add a fixup at this location.
@@ -141,8 +142,8 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   final void fixupAdd (int kind, int offset, Label label)
   {
-    if (label != null && kind != FIXUP_DEFINE
-        && kind != FIXUP_NONE && kind != FIXUP_SWITCH && kind != FIXUP_TRY)
+    if (label != null && kind != FIXUP_DEFINE && kind != FIXUP_DEFINE_UNREACHABLE
+        && kind != FIXUP_SWITCH && kind != FIXUP_TRY)
       label.needsStackMapEntry = true;
     int count = fixup_count;
     if (count == 0)
@@ -160,7 +161,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 	System.arraycopy (fixup_offsets, 0, new_offsets, 0, count);
 	fixup_offsets = new_offsets;
       }
-    fixup_offsets[count] = (offset << 4) | kind;
+    fixupSet(count, kind, offset);
     fixup_labels[count] = label;
     fixup_count = count + 1;
   }
@@ -175,6 +176,13 @@ public class CodeAttr extends Attribute implements AttrContainer
     return fixup_offsets[index] & 15;
   }
 
+    private void fixupSet(int index, int kind, int offset) {
+         fixup_offsets[index] = fixupEncode(kind, offset);
+    }
+    private int fixupEncode(int kind, int offset) {
+        return (offset << 4) | kind;
+    }
+
   /** If true we get a line number entry for each instruction.
    * Normally false, but can be a convenient hack to allow instruction-level
    * stepping/debugging and stacktraces.  In this case {@code LINE==PC}. */
@@ -358,6 +366,13 @@ public class CodeAttr extends Attribute implements AttrContainer
       }
   }
 
+    void setPreviousLabelHere(Label here) {
+        previousLabel = here;
+        boolean[] varsSet = varsSetInCurrentBlock;
+        if (varsSet != null)
+            for (int i = varsSet.length;  --i >= 0; ) varsSet[i] = false;
+    }
+
   public void noteVarType (int offset, Type type)
   {
     int size = type.getSizeInWords();
@@ -968,12 +983,13 @@ public class CodeAttr extends Attribute implements AttrContainer
         pushType(Type.javalanginvokeMethodHandleType);
     }
 
-  public void emitPushNull ()
-  {
-    reserve(1);
-    put1(1);  // aconst_null
-    pushType(Type.nullType);
-  }
+    public void emitPushNull() { emitPushNull(Type.nullType); }
+
+    public void emitPushNull(ObjectType type) {
+        reserve(1);
+        put1(1);  // aconst_null
+        pushType(type);
+    }
 
   /** Push zero or null as appropriate for the given type. */
   public void emitPushDefaultValue (Type type)
@@ -1449,7 +1465,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   }
 
   /** Compile code to get a static field value.
-   * Stack:  ... => ..., value */
+   * Stack: {@code ... => ..., value} */
 
   public final void emitGetStatic(Field field)
   {
@@ -1458,7 +1474,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   }
 
   /** Compile code to get a non-static field value.
-   * Stack:  ..., objectref => ..., value */
+   * Stack: {@code ..., objectref => ..., value} */
 
   public final void emitGetField(Field field)
   {
@@ -1468,7 +1484,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   }
 
   /** Compile code to put a static field value.
-   * Stack:  ..., value => ... */
+   * Stack: {@code ..., value => ...} */
 
   public final void emitPutStatic (Field field)
   {
@@ -1543,8 +1559,10 @@ public class CodeAttr extends Attribute implements AttrContainer
                 var.setType(ctype);
           }
         for (int i = local_types == null ? 0 : local_types.length;  --i >= 0; )
-          if (local_types[i] == t)
-            local_types[i] = ctype;
+          {
+            if (local_types[i] == t)
+                local_types[i] = ctype;
+          }
       }
    if (method.return_type.size != 0)
       pushType(method.return_type);
@@ -1740,7 +1758,6 @@ public class CodeAttr extends Attribute implements AttrContainer
       throw new Error ("non-int type to emitIfCompare1");
     reserve(3);
     emitTransfer (new_if.end_label, opcode);
-    new_if.start_stack_size = SP;
   }
 
   /** Compile start of conditional:  <tt>if (x != 0) ...</tt>.
@@ -1763,7 +1780,13 @@ public class CodeAttr extends Attribute implements AttrContainer
     emitIfCompare1(157); // ifgt
   }
 
-  /** Compile start of a conditional:  <tt>if (!(x opcode null)) ...</tt>.
+  /** Compile start of conditional:  {@code if (x >= 0)}. */
+  public final void emitIfIntGEqZero()
+  {
+    emitIfCompare1(155); // iflt
+  }
+
+  /** Compile start of a conditional:  {@code if (!(x opcode null)) ...}.
    * The value of <tt>x</tt> must already have been pushed and must be of
    * reference type. */
   public final void emitIfRefCompare1 (int opcode)
@@ -1773,22 +1796,21 @@ public class CodeAttr extends Attribute implements AttrContainer
       throw new Error ("non-ref type to emitIfRefCompare1");
     reserve(3);
     emitTransfer (new_if.end_label, opcode);
-    new_if.start_stack_size = SP;
   }  
   
-  /** Compile start of conditional:  if (x != null) */
+  /** Compile start of conditional:  {@code if (x != null) ...}. */
   public final void emitIfNotNull()
   {
     emitIfRefCompare1(198); // ifnull
   }
 
-  /** Compile start of conditional:  if (x == null) */
+  /** Compile start of conditional:  {@code if (x == null) ...} */
   public final void emitIfNull()
   {
     emitIfRefCompare1(199); // ifnonnull
   }  
   
-  /** Compile start of a conditional:  if (!(x OPCODE y)) ...
+  /** Compile start of a conditional:  {@code if (!(x OPCODE y)) ...}
    * The value of x and y must already have been pushed. */
   public final void emitIfIntCompare(int opcode)
   {
@@ -1797,67 +1819,66 @@ public class CodeAttr extends Attribute implements AttrContainer
     popType();
     reserve(3);
     emitTransfer(new_if.end_label, opcode);
-    new_if.start_stack_size = SP;
   }
 
-  /* Compile start of a conditional:  if (x < y) ... */
+  /* Compile start of a conditional:  {@code if (x < y) ...} */
   public final void emitIfIntLt()
   {
     emitIfIntCompare(162);  // if_icmpge
   }
 
+  /* Compile start of a conditional:  {@code if (x >= y) ...} */
+  public final void emitIfIntGEq()
+  {
+    emitIfIntCompare(161);  // if_icmplt
+  }
+
   /** Compile start of a conditional:  if (x != y) ...
    * The values of x and y must already have been pushed. */
   public final void emitIfNEq ()
   {
     IfState new_if = pushIfState();
     emitGotoIfEq(new_if.end_label);
-    new_if.start_stack_size = SP;
   }
 
-  /** Compile start of a conditional:  if (x == y) ...
+  /** Compile start of a conditional:  {@code if (x == y) ...}
    * The values of x and y must already have been pushed. */
   public final void emitIfEq ()
   {
     IfState new_if = pushIfState();
     emitGotoIfNE(new_if.end_label);
-    new_if.start_stack_size = SP;
   }
 
-  /** Compile start of a conditional:  if (x < y) ...
+  /** Compile start of a conditional:  {@code if (x < y) ...}
    * The values of x and y must already have been pushed. */
   public final void emitIfLt ()
   {
     IfState new_if = pushIfState();
     emitGotoIfGe(new_if.end_label);
-    new_if.start_stack_size = SP;
   }
 
-  /** Compile start of a conditional:  if (x >= y) ...
+  /** Compile start of a conditional:  {@code if (x >= y) ...}
    * The values of x and y must already have been pushed. */
   public final void emitIfGe ()
   {
     IfState new_if = pushIfState();
     emitGotoIfLt(new_if.end_label);
-    new_if.start_stack_size = SP;
   }
 
-  /** Compile start of a conditional:  if (x > y) ...
+  /** Compile start of a conditional:  {@code if (x > y) ...}
    * The values of x and y must already have been pushed. */
   public final void emitIfGt ()
   {
     IfState new_if = pushIfState();
     emitGotoIfLe(new_if.end_label);
-    new_if.start_stack_size = SP;
   }
 
-  /** Compile start of a conditional:  if (x <= y) ...
+  /** Compile start of a conditional:  {@code if (x <= y) ...}
    * The values of x and y must already have been pushed. */
   public final void emitIfLe ()
   {
     IfState new_if = pushIfState();
     emitGotoIfGt(new_if.end_label);
-    new_if.start_stack_size = SP;
   }
 
   /** Emit a 'ret' instruction.
@@ -1882,7 +1903,6 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public final void emitThen()
   {
-    if_stack.start_stack_size = SP;
   }
 
   public final void emitIfThen ()
@@ -1898,23 +1918,10 @@ public class CodeAttr extends Attribute implements AttrContainer
       {
         Label end_label = new Label (this);
         if_stack.end_label = end_label;
-	int growth = SP-if_stack.start_stack_size;
-	if_stack.stack_growth = growth;
-	if (growth > 0)
-	  {
-	    if_stack.then_stacked_types = new Type[growth];
-	    System.arraycopy (stack_types, if_stack.start_stack_size,
-			      if_stack.then_stacked_types, 0, growth);
-	  }
-	else
-	  if_stack.then_stacked_types = new Type[0];  // ???
 	emitGoto (end_label);
       }
     else
       if_stack.end_label = null;
-    while (SP > if_stack.start_stack_size)
-      popType();
-    SP = if_stack.start_stack_size;
     if (else_label != null)
       else_label.define (this);
     if_stack.doing_else = true;    
@@ -1923,31 +1930,6 @@ public class CodeAttr extends Attribute implements AttrContainer
   /** Compile end of conditional. */
   public final void emitFi ()
   {
-    if (! if_stack.doing_else)
-      { // There was no 'else' clause.
-	if (reachableHere ()
-	    && SP != if_stack.start_stack_size)
-	  throw new Error("at PC "+PC+" then clause grows stack with no else clause");
-      }
-    else if (if_stack.then_stacked_types != null)
-      {
-	int then_clause_stack_size
-	  = if_stack.start_stack_size + if_stack.stack_growth;
-	if (! reachableHere ())
-	  {
-	    if (if_stack.stack_growth > 0)
-	      System.arraycopy (if_stack.then_stacked_types, 0,
-				stack_types, if_stack.start_stack_size,
-				if_stack.stack_growth);
-	    SP = then_clause_stack_size;
-	  }
-	else if (SP != then_clause_stack_size)
-	  throw new Error("at PC "+PC+": SP at end of 'then' was " +
-			  then_clause_stack_size
-			  + " while SP at end of 'else' was " + SP);
-        setReachable(true);
-      }
-
     if (if_stack.end_label != null)
       if_stack.end_label.define (this);
     // Pop the if_stack.
@@ -1979,26 +1961,62 @@ public class CodeAttr extends Attribute implements AttrContainer
         return new IfState(this);
     }
 
-  public final void emitConvert (Type from, Type to)
-  {
-    String to_sig = to.getSignature();
-    String from_sig = from.getSignature();
-    int op = -1;
-    if (to_sig.length() == 1 || from_sig.length() == 1)
-      {
+    public final void fixUnsigned(Type stackType) {
+        if (stackType instanceof PrimType
+            && ((PrimType) stackType).isUnsigned()) {
+            char sig1 = stackType.getSignature().charAt(0);
+            if (sig1 == 'S') {
+                reserve(1);
+                put1(146); // i2c
+            } else if (sig1 == 'B') {
+                emitPushInt(255);
+                emitAnd();
+            }
+        }
+    }
+
+    public final void emitConvert(PrimType from, PrimType to) {
+        String to_sig = to.getSignature();
+        String from_sig = from.getSignature();
+        int op = -1;
 	char to_sig0 = to_sig.charAt(0);
 	char from_sig0 = from_sig.charAt(0);
 	if (from_sig0 == to_sig0)
-	  return;
+            return;
 	if (from.size < 4)
 	  from_sig0 = 'I';
-	if (to.size < 4)
-	  {
+	if (to.size < 4) {
 	    emitConvert(from, Type.intType);
 	    from_sig0 = 'I';
-	  }
+            if (to.isUnsigned()) {
+                if (to_sig0 == 'S')
+                    to_sig0 = 'C';
+                else if (to_sig0 == 'B') {
+                    emitPushInt(0xff);
+                    emitAnd();
+                    return;
+                }
+            }
+        }
+        if (from_sig0 == 'J' && from.isUnsigned()
+            && (to_sig0 == 'F' || to_sig0 == 'D')) {
+            emitPushInt(1);
+            emitUshr();
+            emitConvert(Type.longType, to);
+            emitPushConstant(2, to);
+            emitMul();
+            return;
+        }
+        if (from_sig0 == 'I' && from.isUnsigned()
+            && (to_sig0 == 'J' || to_sig0 == 'F' || to_sig0 == 'D')) {
+            emitConvert(Type.intType, Type.longType);
+            reserve(4);
+            emitPushLong(0xffffffffL);
+            emitAnd();
+            from_sig0 = 'J';
+        }
 	if (from_sig0 == to_sig0)
-	  return;
+            return;
 	switch (from_sig0)
 	  {
 	  case 'I':
@@ -2037,14 +2055,13 @@ public class CodeAttr extends Attribute implements AttrContainer
 	      }
 	    break;
 	  }
-      }
-    if (op < 0)
-      throw new Error ("unsupported CodeAttr.emitConvert");
-    reserve(1);
-    popType();
-    put1(op);
-    pushType(to);
-  }
+        if (op < 0)
+            throw new Error ("unsupported CodeAttr.emitConvert");
+        reserve(1);
+        popType();
+        put1(op);
+        pushType(to);
+    }
 
   private void emitCheckcast (Type type, int opcode)
   {
@@ -2190,6 +2207,7 @@ public class CodeAttr extends Attribute implements AttrContainer
     handler.stackTypes[0] = handler_class;
     setTypes(handler);
     fixupAdd(FIXUP_TRY_HANDLER, 0, handler);
+    setReachable(true);
   }
 
   /** Beginning of code that has a cleanup handler.
@@ -2202,7 +2220,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   public void emitWithCleanupStart ()
   {
     int savedSP = SP;
-    SP = 0;
+    SP = 0; // Hack to disable emitTryStart needlessly saving the stack.
     emitTryStart(false, null);
     SP = savedSP;
   }
@@ -2212,20 +2230,8 @@ public class CodeAttr extends Attribute implements AttrContainer
    */
   public void emitWithCleanupCatch (Variable catchVar)
   {
-    emitTryEnd();
-    Type[] savedTypes;
-    if (SP > 0)
-      {
-        savedTypes = new Type[SP];
-        System.arraycopy(stack_types, 0, savedTypes, 0, SP);
-        SP = 0;
-      }
-    else
-      savedTypes = null;
-    try_stack.savedTypes = savedTypes;
-
+    emitTryEnd(false);
     try_stack.saved_result = catchVar;
-    int save_SP = SP;
     emitCatchStart(catchVar);
   }
 
@@ -2239,18 +2245,7 @@ public class CodeAttr extends Attribute implements AttrContainer
       emitLoad(catchVar);
     emitThrow();
     emitCatchEnd();
-    Type[] savedTypes = try_stack.savedTypes;
     emitTryCatchEnd();
-    if (savedTypes != null)
-      {
-        SP = savedTypes.length;
-        if (SP >= stack_types.length)
-          stack_types = savedTypes;
-        else
-          System.arraycopy(savedTypes, 0, stack_types, 0, SP);
-      }
-    else
-      SP = 0;
   }
 
 
@@ -2299,10 +2294,10 @@ public class CodeAttr extends Attribute implements AttrContainer
       try_state.finally_subr = new Label();
   }
 
-  public void emitTryEnd ()
-  {
-    emitTryEnd(false);
-  }
+    @Deprecated
+    public void emitTryEnd() {
+    }
+
   private void emitTryEnd (boolean fromFinally)
   {
     if (try_stack.tryClauseDone)
@@ -2314,18 +2309,24 @@ public class CodeAttr extends Attribute implements AttrContainer
     try_stack.end_try = getLabel();
   }
 
-  public void emitCatchStart(Variable var)
+    public void emitCatchStart(Variable var) {
+        if (var == null)
+            emitCatchStart((ClassType) null);
+        else {
+            emitCatchStart((ClassType) var.getType());
+            emitStore(var);
+        }
+    }
+
+  public void emitCatchStart(ClassType type)
   {
     emitTryEnd(false);
     setTypes(try_stack.start_try.localTypes, Type.typeArray0);
     if (try_stack.try_type != null)
       emitCatchEnd();
-    ClassType type = var == null ? null : (ClassType) var.getType();
     try_stack.try_type = type;
     addHandler(try_stack.start_try, try_stack.end_try, type);
     setReachable(true);
-    if (var != null)
-      emitStore(var);
   }
 
   public void emitCatchEnd()
@@ -2370,7 +2371,7 @@ public class CodeAttr extends Attribute implements AttrContainer
     if (useJsr())
       {
         SP = 0;
-        emitCatchStart(null);
+        emitCatchStart((ClassType) null);
         emitStore(try_stack.exception);
         emitJsr(try_stack.finally_subr);
         emitLoad(try_stack.exception);
@@ -2378,9 +2379,8 @@ public class CodeAttr extends Attribute implements AttrContainer
       }
     else
       {
-        setUnreachable();
-        Label endLabel = new Label(this);
-        int fragment_cookie = beginFragment(endLabel);
+        if (reachableHere())
+          emitGoto(try_stack.finally_subr);
         addHandler(try_stack.start_try, try_stack.end_try, Type.javalangThrowableType);
         if (try_stack.saved_result != null)
           emitStoreDefaultValue(try_stack.saved_result);
@@ -2389,8 +2389,6 @@ public class CodeAttr extends Attribute implements AttrContainer
             emitPushInt(-1);  // Return switch case code.
             emitSwap();
           }
-        emitGoto(try_stack.finally_subr);
-        endFragment(fragment_cookie);
       }
     try_stack.finally_subr.define(this);
     
@@ -2410,8 +2408,10 @@ public class CodeAttr extends Attribute implements AttrContainer
   }
 
   public void emitFinallyEnd()
-  { 
-    if (useJsr())
+  {
+    if (! reachableHere())
+      try_stack.end_label = null;
+    else if (useJsr())
       emitRet(try_stack.finally_ret_addr);
     else if (try_stack.end_label == null && try_stack.exitCases == null)
       {
@@ -2547,21 +2547,23 @@ public class CodeAttr extends Attribute implements AttrContainer
             return;
 
         // For each label, set it to its maximum limit, assuming all
-        // fixups causes the code the be expanded.  We need a prepass
+        // fixups causes the code to be expanded.  We need a prepass
         // for this, since FIXUP_MOVEs can cause code to be reordered.
         // Also, convert each FIXUP_MOVE_TO_END to FIXUP_MOVE.
 
         int delta = 0;
         int instruction_tail = fixup_count;
-        fixupAdd(CodeAttr.FIXUP_MOVE, 0, null);
+        fixupAdd(CodeAttr.FIXUP_MOVE, -1, null);
 
-        /* DEBUGGING:
-        System.err.println("processFixups for "+getMethod());
-        ClassTypeWriter writer =
-            new ClassTypeWriter(getMethod().getDeclaringClass(), System.err, 0);
-        writer.println("processFixups for "+getMethod());
-        disAssembleWithFixups(writer);
-        writer.flush();
+        /* DEBUGGING
+        if (false) {
+            ClassTypeWriter writer =
+                new ClassTypeWriter(getMethod().getDeclaringClass(),
+                                    System.err, 0);
+            writer.println("processFixups1 for "+getMethod());
+            disAssembleWithFixups(writer);
+            writer.flush();
+        }
         */
 
   loop1:
@@ -2571,6 +2573,27 @@ public class CodeAttr extends Attribute implements AttrContainer
 	int kind = offset & 15;
 	offset >>= 4;
 	Label label = fixup_labels[i];
+        // Optimize: TRANSFER L1; ...; L1: GOTO L2
+        // to: TRANSFER L2; ...; L1: GOTO L2
+        // If L1 is a FIXUP_DEFINE_UNREACHABLE then we can delete the GOTO L2
+        // (replace it with a DELETE3), since L1 is never reached.
+        if (kind >= FIXUP_CASE && kind <= FIXUP_TRANSFER2) {
+            int max = fixup_count;
+        goto_to_goto:
+            while (label != null && --max >= 0) {
+                int labpc = fixupOffset(label.first_fixup);
+                for (int def = label.first_fixup+1; ; def++) {
+                    if (def >= fixup_count || labpc != fixupOffset(def)) {
+                        break goto_to_goto;
+                    } else if (fixupKind(def) == FIXUP_GOTO
+                               || fixupKind(def) == FIXUP_DELETE3) {
+                        label = fixup_labels[def];
+                        fixup_labels[i] = label;
+                        break;
+                    }
+                }
+            }
+        }
 	switch (kind)
 	  {
 	  case FIXUP_TRY:
@@ -2578,10 +2601,33 @@ public class CodeAttr extends Attribute implements AttrContainer
             break;
 	  case FIXUP_LINE_PC:
 	    i++;
-	  case FIXUP_NONE:
 	  case FIXUP_CASE:
 	  case FIXUP_DELETE3:
 	    break;
+          case FIXUP_DEFINE_UNREACHABLE:
+              // If we're currently "unreachable" and we see
+              //    L0: L1: ... Ln: GOTO Lg
+              // then we can delete the "GOTO Lg" because we also (separately)
+              // patch each use of Li to Lg.
+              while (i + 1 < fixup_count && fixupKind(i+1) == FIXUP_DEFINE
+                     && fixupOffset(i+1) == offset) {
+                  i++;
+                  label.position += delta;
+                  label = fixup_labels[i];
+              }
+              if (i + 1 < fixup_count && fixupKind(i+1) == FIXUP_GOTO
+                  && fixupOffset(i+1) == offset) {
+                  for (int j = i; ; j--) {
+                      fixup_labels[j].needsStackMapEntry = false;
+                      if (fixupKind(j) == FIXUP_DEFINE_UNREACHABLE)
+                          break;
+                  }
+                  i++;
+                  fixupSet(i, FIXUP_DELETE3, offset);
+                  delta -= 3;
+                  continue;
+              }
+              // fall through
 	  case FIXUP_DEFINE:
 	    label.position += delta;
 	    break;
@@ -2589,13 +2635,10 @@ public class CodeAttr extends Attribute implements AttrContainer
 	    delta += 3;  // May need to add up to 3 padding bytes.
 	    break;
 	  case FIXUP_GOTO:
-	    // The first test fails in this case:  GOTO L2; L1: L2:  FIXME
-	    if (label.first_fixup == i + 1
-		&& fixupOffset(i+1) == offset + 3)
+            if (fixupOffset(label.first_fixup) == offset + 3)
 	      {
 		// Optimize: GOTO L; L:
-		fixup_offsets[i] = (offset << 4) | FIXUP_DELETE3;
-		fixup_labels[i] = null;
+                fixupSet(i, FIXUP_DELETE3, offset);
 		delta -= 3;
 		break;
 	      }
@@ -2613,9 +2656,9 @@ public class CodeAttr extends Attribute implements AttrContainer
 	    instruction_tail = offset;
 	    // ... fall through ...
 	  case FIXUP_MOVE:
-	    int cur_pc = ((i+1) >= fixup_count ? PC
-			  : fixupOffset(fixup_labels[i+1].first_fixup));
-	    fixup_offsets[i] = (cur_pc << 4) | FIXUP_MOVE;
+            int cur_pc = ((i+1) >= fixup_count ? PC
+                          : fixupOffset(fixup_labels[i+1].first_fixup));
+            fixupSet(i, FIXUP_MOVE, cur_pc);
 	    if (label == null)
 	      break loop1;
 	    else
@@ -2645,19 +2688,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 	Label label = fixup_labels[i];
 	if (label != null && label.position < 0)
 	  throw new Error ("undefined label "+label);
-	while (label != null
-	       && kind >= FIXUP_GOTO && kind <= FIXUP_TRANSFER2
-	       && label.first_fixup + 1 < fixup_count
-	       && (fixup_offsets[label.first_fixup + 1]
-		   == ((fixup_offsets[label.first_fixup] & 15) | FIXUP_GOTO)))
-	  {
-	    // Optimize  JUMP L; ... L:  GOTO X
-	    // (where the JUMP is any GOTO or other transfer)
-	    // by changing the JUMP L to JUMP X.
-	    label = fixup_labels[label.first_fixup + 1];
-	    fixup_labels[i] = label;
-	  }
-	offset = offset >> 4;
+        offset = offset >> 4;
 	switch (kind)
 	  {
 	  case FIXUP_TRY:
@@ -2666,13 +2697,13 @@ public class CodeAttr extends Attribute implements AttrContainer
             break;
 	  case FIXUP_LINE_PC:
 	    i++;
-	  case FIXUP_NONE:
 	  case FIXUP_CASE:
 	    break;
 	  case FIXUP_DELETE3:
 	    delta -= 3;
 	    new_size -= 3;
 	    break;
+          case FIXUP_DEFINE_UNREACHABLE:
 	  case FIXUP_DEFINE:
 	    label.position = offset + delta;
 	    break;
@@ -2687,7 +2718,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 	    int rel = label.position - (offset+delta);
 	    if ((short) rel == rel)
 	      {
-		fixup_offsets[i] = (offset << 4) | FIXUP_TRANSFER2;
+                fixupSet(i, FIXUP_TRANSFER2, offset);
 	      }
 	    else
 	      {
@@ -2711,6 +2742,16 @@ public class CodeAttr extends Attribute implements AttrContainer
 	i++;
       }
 
+    /* DEBUGGING
+    if (false) {
+        ClassTypeWriter writer =
+            new ClassTypeWriter(getMethod().getDeclaringClass(), System.err, 0);
+        writer.println("processFixups3 for "+getMethod());
+        disAssembleWithFixups(writer);
+        writer.flush();
+    }
+    */
+
     byte[] new_code = new byte[new_size];
     int prev_linenumber = -1;
     int new_pc = 0;
@@ -2738,10 +2779,9 @@ public class CodeAttr extends Attribute implements AttrContainer
               throw new Error("labels out of order");
 	    switch (kind)
 	      {
-	      case FIXUP_NONE:
-		break;
 	      case FIXUP_DEFINE:
-                if (stackMap != null && label != null && label.stackTypes != null && label.needsStackMapEntry)
+              case FIXUP_DEFINE_UNREACHABLE:
+                  if (stackMap != null && label != null && label.isUsed() && label.needsStackMapEntry)
                   {
                     pendingStackMapLabel
                       = mergeLabels(pendingStackMapLabel, label);
@@ -2975,11 +3015,11 @@ public class CodeAttr extends Attribute implements AttrContainer
             dst.print(" @");  dst.print(offset);
             prev_pc = pc;
             switch (kind) {
-            case FIXUP_NONE:
-                dst.println(" NONE");
-                break;
             case FIXUP_DEFINE:
+            case FIXUP_DEFINE_UNREACHABLE:
                 dst.print(" DEFINE ");
+                if (kind == FIXUP_DEFINE_UNREACHABLE)
+                    dst.print("(unreachable) ");
                 dst.println(label);
                 break;
             case FIXUP_SWITCH:
@@ -3021,6 +3061,8 @@ public class CodeAttr extends Attribute implements AttrContainer
                 dst.print(" type: ");
                 dst.println(fixup_offsets[i] >> 4);
                 i++;
+                dst.print(" - handler: ");
+                dst.println(fixup_labels[i]);
                 break;
             case FIXUP_LINE_PC:
                 dst.print(" LINE ");
@@ -3177,7 +3219,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 		  {
 		    int index;
 		    dst.print("ret ");
-		    if (wide) { index = readUnsignedShort(i); index += 2; }
+		    if (wide) { index = readUnsignedShort(i); i += 2; }
 		    else { index = code[i] & 0xff; i++; }
 		    wide = false;
 		    dst.print(index);
@@ -3370,9 +3412,11 @@ public class CodeAttr extends Attribute implements AttrContainer
    */
   public void endFragment (int cookie)
   {
-    fixup_offsets[cookie] = (fixup_count << 4) | FIXUP_MOVE_TO_END;
+    fixupSet(cookie, FIXUP_MOVE_TO_END, fixup_count);
     Label after = fixup_labels[cookie];
-    fixupAdd(FIXUP_MOVE, 0, null);
+    fixupAdd(FIXUP_MOVE, -1, null);
     after.define(this);
+    int fx = fixup_count - 1;
+    fixupSet(fx, FIXUP_DEFINE, fixupOffset(fx));
   }
 }
