@@ -249,6 +249,15 @@ public abstract class Type
     return size < 4 ? intType : this;
   }
 
+  public Type promoteIfUnsigned() {
+      if (this instanceof PrimType) {
+          char sig1 = signature.charAt(0);
+          if ((sig1 == 'B' || sig1 == 'S') && ((PrimType) this).isUnsigned())
+              return intType;
+      }
+      return this;
+  }
+
   public final int getSize() { return size; }
   public int getSizeInWords () { return size > 4 ? 2 : 1; }
 
@@ -455,6 +464,8 @@ public abstract class Type
     }
 
     public static int isCompatibleWithValue(Type targetType, Type valueType) {
+        if (targetType == valueType)
+            return 2;
         int comp = targetType.compare(valueType);
         return comp >= 0 ? 1 : comp == -3 ? -1 : 0;
     }
@@ -550,6 +561,11 @@ public abstract class Type
     return true;
   }
 
+    public static boolean isSame(Type t1, Type t2) {
+        return t1 == t2
+            || (t1 != null && t2 != null && t1.equals(t2));
+    }
+
   public void emitIsInstance (CodeAttr code)
   {
     code.emitInstanceof(this);
@@ -689,13 +705,13 @@ public abstract class Type
     public static final ObjectType nullType
     //= new ObjectType("(type of null)");
     = new SpecialObjectType("(type of null)", objectType);
-    public static final ObjectType errorType = new ObjectType("(error type)");
+    public static final ObjectType errorType
+        = new SpecialObjectType("<error-type>", javalangObjectType);
 
     static public ClassType javalangStringType = ClassType.make("java.lang.String");
     /* The String type. but coercion is handled by toString. */
     public static final ObjectType toStringType
         = new SpecialObjectType("String", javalangStringType);
-    //= new ClassType("java.lang.String");
 
   /** @deprecated */
   public static final ClassType pointer_type = javalangObjectType;
