@@ -104,17 +104,17 @@
   (instance? x <gnu.lists.SimpleVector>))
 
 (define (aref
-         (array <gnu.lists.SimpleVector>)
-         (k <int>))
+         (array ::gnu.lists.SimpleVector)
+         (k ::int))
   (invoke array 'get k))
 
-(define (aset (array <gnu.lists.SimpleVector>)
-	      (k <int>)
+(define (aset (array ::gnu.lists.SimpleVector)
+	      (k ::int)
 	      obj)
   (invoke array 'set k obj)
   obj)
 
-(define (fillarray (array <gnu.lists.SimpleVector>) obj)
+(define (fillarray (array ::gnu.lists.SimpleVector) obj)
   (invoke array 'fill obj)
   obj)
 
