@@ -28,6 +28,7 @@ public class ApplyExp extends Expression
   public static final int TAILCALL = NEXT_AVAIL_FLAG;
   public static final int INLINE_IF_CONSTANT = NEXT_AVAIL_FLAG << 1;
   public static final int MAY_CONTAIN_BACK_JUMP = NEXT_AVAIL_FLAG << 2;
+  public static final int IS_SUPER_INIT = NEXT_AVAIL_FLAG << 3;
 
   /** Containing LambdaExp. */
   LambdaExp context;
@@ -189,6 +190,10 @@ public class ApplyExp extends Expression
   public void compile (Compilation comp, Target target)
   {
     compile(this, comp, target, true);
+    if (getFlag(IS_SUPER_INIT)) {
+        ((ClassExp) comp.currentScope().currentLambda().getOuter())
+            .compileCallInitMethods(comp);
+    }
   }
 
   public static void compile (ApplyExp exp, Compilation comp, Target target)
@@ -398,7 +403,6 @@ public class ApplyExp extends Expression
 	&& (exp.isTailCall() || target instanceof ConsumerTarget)
 	&& ! comp.curLambda.getInlineOnly())
       {
-	ClassType typeContext = Compilation.typeCallContext;
 	exp_func.compile(comp, new StackTarget(Compilation.typeProcedure));
 	// evaluate args to frame-locals vars;  // may recurse!
 	if (args_length <= 4 && exp.isSimple())
@@ -421,21 +425,7 @@ public class ApplyExp extends Expression
 	    code.emitInvoke(Compilation.typeProcedure
 			    .getDeclaredMethod("checkN", 2));
 	  }
-	if (exp.isTailCall())
-	  {
-	    code.emitReturn();
-	  }
-	else if (((ConsumerTarget) target).isContextTarget())
-	  {
-	    comp.loadCallContext();
-	    code.emitInvoke(typeContext.getDeclaredMethod("runUntilDone", 0));
-	  }
-	else
-	  {
-	    comp.loadCallContext();
-	    code.emitLoad(((ConsumerTarget) target).getConsumerVariable());
-	    code.emitInvoke(typeContext.getDeclaredMethod("runUntilValue", 1));
-	  }
+        finishTrampoline(exp.isTailCall(), target, comp);
 	return;
       }
 
@@ -512,6 +502,22 @@ public class ApplyExp extends Expression
     target.compileFromStack(comp, Type.pointer_type);
   }
 
+    static void finishTrampoline(boolean isTailCall, Target target, Compilation comp) {
+        CodeAttr code = comp.getCode();
+        ClassType typeContext = Compilation.typeCallContext;
+        if (isTailCall) {
+            code.emitReturn();
+        } else if (target instanceof IgnoreTarget
+                   || ((ConsumerTarget) target).isContextTarget()) {
+            comp.loadCallContext();
+            code.emitInvoke(typeContext.getDeclaredMethod("runUntilDone", 0));
+        } else {
+            comp.loadCallContext();
+            code.emitLoad(((ConsumerTarget) target).getConsumerVariable());
+            code.emitInvoke(typeContext.getDeclaredMethod("runUntilValue", 1));
+        }
+    }
+
   public Expression deepCopy (gnu.kawa.util.IdentityHashTable mapper)
   {
     Expression f = deepCopy(func, mapper);
@@ -529,10 +535,37 @@ public class ApplyExp extends Expression
   }
 
     public void visitArgs(InlineCalls visitor) {
+        visitArgs(visitor, null);
+    }
+    public void visitArgs(InlineCalls visitor, LambdaExp lexp) {
         int nargs = args.length;
-        args = visitor.visitExps(args, nargs,
-                                 isAppendValues() ? null
-                                 : InlineCalls.ValueNeededType.instance);
+        Type dtype = isAppendValues() ? null
+            : InlineCalls.ValueNeededType.instance;
+        // For simplicity, for now.
+        Declaration param = lexp == null
+            || firstKeywordArgIndex != 0
+            || lexp.keywords != null
+            ? null
+            : lexp.firstDecl();
+        for (int i = 0;  i < nargs && visitor.getExitValue() == null;  i++) {
+            while (param != null
+                   && (param.isThisParameter()
+                       || param.getFlag(Declaration.PATTERN_NESTED)))
+                param = param.nextDecl();
+            Type ptype = dtype;
+            if (param != null && i < lexp.min_args+lexp.opt_args
+                && (firstSpliceArg < 0 || i > firstSpliceArg))
+                ptype = param.getType();
+            else if (param != null
+                     && param.getFlag(Declaration.IS_REST_PARAMETER)) {
+                Type vtype = param.getType();
+                if (vtype instanceof ArrayType)
+                    ptype = ((ArrayType) vtype).getComponentType();
+            }
+            args[i] = visitor.visitAndUpdate(args[i], ptype);
+            if (param != null && ! param.getFlag(Declaration.IS_REST_PARAMETER))
+                param = param.nextDecl();
+        }
     }
 
   protected <R,D> void visitChildren(ExpVisitor<R,D> visitor, D d)
