@@ -1009,7 +1009,7 @@ public class LambdaExp extends ScopeExp {
             = (getFlag(SEQUENCE_RESULT)
                || getCallConvention () >= Compilation.CALL_WITH_CONSUMER)
             ? Type.voidType
-            : getReturnType().getImplementationType();
+            : getReturnType().promoteIfUnsigned().getImplementationType();
         int extraArg = (closureEnvType != null && closureEnvType != ctype) ? 1 : 0;
 
         String rtypeEnc = comp.getLanguage().encodeType(getReturnType());
@@ -1036,7 +1036,8 @@ public class LambdaExp extends ScopeExp {
             if (var != null && var.isThisParameter())
                 var = var.nextDecl();
             for (int itype = 0; itype < plainArgs; var = var.nextDecl()) {
-                atypes[extraArg + itype++] = var.getType().getImplementationType();
+                atypes[extraArg + itype++] =
+                    var.getType().promoteIfUnsigned().getImplementationType();
                 String encType = comp.getLanguage().encodeType(var.getType());
                 if (encType == null /* || not interesting */)
                     encType = "";
@@ -1246,7 +1247,7 @@ public class LambdaExp extends ScopeExp {
             if (state>=Compilation.COMPILED && state != Compilation.ERROR_SEEN)
                 comp.error('f', "internal error - allocate method for "+this
                            +" in module "+currentModule()
-                           +" that has already been compiled");
+                           +" that has already been compiled\n(Try removing all class files and doing a full re-compile.)");
         }
         if (! getNeedsClosureEnv())
             closureEnvType = null;
@@ -1634,7 +1635,7 @@ public class LambdaExp extends ScopeExp {
                     Expression arg;
                     String lastTypeName = restArgType.getName();
                     if ("gnu.lists.LList".equals(lastTypeName))
-                        arg = new QuoteExp(gnu.lists.LList.Empty);
+                        arg = QuoteExp.emptyExp;
                     else if ("java.lang.Object[]".equals(lastTypeName))
                         arg = new QuoteExp(Values.noArgs);
                     else // FIXME
@@ -1686,8 +1687,11 @@ public class LambdaExp extends ScopeExp {
         }
         else
             target = Target.pushValue(getReturnType());
+        ScopeExp savedScope = comp.currentScope();
+        comp.current_scope = this;
         body.compileWithPosition(comp, target,
                                  body.getLineNumber() > 0 ? body : this);
+        comp.current_scope = savedScope;
         comp.callContextVar = callContextSave;
     }
 
@@ -1793,7 +1797,7 @@ public class LambdaExp extends ScopeExp {
             if (inlined != null)
                 return visitor.visit(inlined, required);
         }
-        exp.visitArgs(visitor);
+        exp.visitArgs(visitor, this);
         int args_length = exp.args.length;
         int spliceCount = exp.spliceCount();
         int nonSpliceCount = args_length - spliceCount;
@@ -1803,57 +1807,6 @@ public class LambdaExp extends ScopeExp {
                                                   nonSpliceCount);
         if (msg != null)
             return visitor.noteError(msg);
-        int conv = getCallConvention();
-        Compilation comp = visitor.getCompilation();
-        Method method;
-        // Mostly duplicates logic with ApplyExp.compile.
-        if (comp.inlineOk(this) && isClassMethod()
-            && (conv <= Compilation.CALL_WITH_CONSUMER
-                || (conv == Compilation.CALL_WITH_TAILCALLS))
-            && (method = getMethod(nonSpliceCount, spliceCount)) != null) {
-            // This is an optimization to expand a call to a method in the
-            // same ClassExp.  The result is a call to a PrimProcedure instead.
-            // This isn't just an optimization, since the re-write is
-            // needed to ensure that if we're in an inner lambda that the
-            // $this$ declaration is captured in a closure.  (See the
-            // 'new ThisExp(d)' below.)  Otherwise, we could could defer this
-            // optimization to ApplyExp.compile.  (Conversely, we can't do the
-            // latter optimization here instead, because we may not have called
-            // addMethodFor yet - since (except in the case of class methods)
-            // that happens later, after FindCapturedVars.  Yuck.)
-            boolean isStatic = nameDecl.isStatic();
-            if (! isStatic && getOuter() instanceof ClassExp) {
-                ClassExp cl = (ClassExp) getOuter();
-                if (cl.isMakingClassPair()) {
-                }
-            }
-            PrimProcedure mproc = new PrimProcedure(method, this);
-            Expression[] margs;
-            if (isStatic)
-                margs = exp.args;
-            else {
-                LambdaExp curLambda = visitor.getCurrentLambda();
-                for (;;) {
-                    if (curLambda == null)
-                        return visitor.noteError("internal error: missing "+this);
-                    if (curLambda.getOuter() == getOuter()) // I.e. same class.
-                        break;
-                    curLambda = curLambda.outerLambda();
-                }
-                Declaration d = curLambda.firstDecl();
-                if (d==null || ! d.isThisParameter())
-                    return visitor.noteError("calling non-static method "
-                                             +getName()+" from static method "
-                                             +curLambda.getName());
-                int nargs = exp.getArgCount();
-                margs = new Expression[1 + nargs];
-                System.arraycopy(exp.getArgs(), 0, margs, 1, nargs);
-                margs[0] = new ThisExp(d);
-            }
-            ApplyExp nexp = new ApplyExp(mproc, margs);
-            nexp.adjustSplice(exp, isStatic ? 0 : 1);
-            return nexp.setLine(exp);
-        }
         return exp;
     }
 
@@ -2015,12 +1968,17 @@ public class LambdaExp extends ScopeExp {
      */
     public Expression getBodyFirstExpression() {
         Expression bodyFirst = body;
-        while (bodyFirst instanceof BeginExp) {
-            BeginExp bbody = (BeginExp) bodyFirst;
-            if (bbody.length == 0)
-                bodyFirst = null;
-            else
-                bodyFirst = bbody.exps[0];
+        for (;;) {
+            if (bodyFirst instanceof BeginExp) {
+                BeginExp bbody = (BeginExp) bodyFirst;
+                if (bbody.length == 0)
+                    bodyFirst = null;
+                else
+                    bodyFirst = bbody.exps[0];
+            } else if (bodyFirst instanceof LetExp) {
+                bodyFirst = ((LetExp) bodyFirst).getBody();
+            } else
+                break;
         }
         return bodyFirst;
     }
