@@ -9,9 +9,12 @@ import gnu.lists.*;
 import gnu.text.Lexer;
 import gnu.text.SourceMessages;
 import gnu.kawa.io.CharArrayInPort;
+import gnu.kawa.io.CheckConsole;
 import gnu.kawa.io.InPort;
 import gnu.kawa.io.OutPort;
+import gnu.kawa.io.TtyInPort;
 import gnu.kawa.reflect.*;
+import gnu.kawa.format.AbstractFormat;
 import gnu.kawa.functions.GetNamedPart;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
@@ -45,6 +48,10 @@ public abstract class Language
   public static void setCurrentLanguage (Language language)
   {
     current.set(language);
+    if (CheckConsole.prompt1.get(null) == null)
+        CheckConsole.prompt1.set(language.getPrimaryPrompt());
+    if (CheckConsole.prompt2.get(null) == null)
+        CheckConsole.prompt2.set(language.getSecondaryPrompt());
   }
 
   public static Language setSaveCurrent (Language language)
@@ -489,8 +496,7 @@ public abstract class Language
   {
     OutPort oport = out instanceof OutPort ? (OutPort) out
       : new OutPort(out);
-    oport.objectFormat = getFormat(false);
-    return oport;
+    return getFormat(false).makeConsumer(oport);
   }
 
   public String getName()
@@ -681,7 +687,36 @@ public abstract class Language
       }
   }
 
-  /** Flag to tell parse that expression will be evaluated immediately.
+    public final Compilation getCompilation(Lexer lexer, int options, ModuleInfo info) {
+        SourceMessages messages = lexer.getMessages();
+        NameLookup lexical = ((options & PARSE_CURRENT_NAMES) != 0
+                              ? NameLookup.getInstance(getEnvironment(), this)
+                              : new NameLookup(this));
+        boolean immediate = (options & PARSE_IMMEDIATE) != 0;
+        Compilation tr = getCompilation(messages, lexical);
+        if (requirePedantic)
+            tr.setPedantic(true);
+        if (! immediate)
+            tr.mustCompile = true;
+        tr.immediate = immediate;
+        tr.langOptions = options;
+        if ((options & PARSE_EMIT_MAIN) != 0)
+            tr.currentOptions.set("main", null);
+        if ((options & PARSE_EXPLICIT) != 0)
+            tr.explicit = true;
+        if ((options & PARSE_PROLOG) != 0)
+            tr.setState(Compilation.PROLOG_PARSING);
+        ModuleExp module = tr.pushNewModule(lexer);
+        if ((options & PARSE_INTERACTIVE_MODULE) != 0) {
+            tr.setInteractiveName();
+            module.setFlag(ModuleExp.INTERACTIVE);
+        }
+        if (info != null)
+            info.setCompilation(tr);
+        return tr;
+    }
+
+ /** Flag to tell parse that expression will be evaluated immediately.
    * I.e. we're not creating class files for future execution. */
   public static final int PARSE_IMMEDIATE = 1;
   /** Flag to tell parse to use current NameLookup.
@@ -720,14 +755,6 @@ public abstract class Language
     return parse(getLexer(port, messages), options, null);
   }
 
-  public final Compilation parse(InPort port,
-                                 gnu.text.SourceMessages messages,
-                                 ModuleInfo info)
-    throws java.io.IOException, gnu.text.SyntaxException
-  {
-    return parse(getLexer(port, messages), Language.PARSE_PROLOG, info);
-  }
-
   public final Compilation parse(InPort port,
                                  gnu.text.SourceMessages messages,
                                  int options, ModuleInfo info)
@@ -737,41 +764,25 @@ public abstract class Language
   }
 
   public final Compilation parse(Lexer lexer, int options, ModuleInfo info)
-    throws java.io.IOException, gnu.text.SyntaxException
-  {
-    SourceMessages messages = lexer.getMessages();
-    NameLookup lexical = ((options & PARSE_CURRENT_NAMES) != 0
-                          ? NameLookup.getInstance(getEnvironment(), this)
-                          : new NameLookup(this));
-    boolean immediate = (options & PARSE_IMMEDIATE) != 0;
-    Compilation tr = getCompilation(messages, lexical);
-    if (requirePedantic)
-      tr.setPedantic(true);
-    if (! immediate)
-      tr.mustCompile = true;
-    tr.immediate = immediate;
-    tr.langOptions = options;
-    if ((options & PARSE_EMIT_MAIN) != 0)
-      tr.currentOptions.set("main", null);
-    if ((options & PARSE_EXPLICIT) != 0)
-      tr.explicit = true;
-    if ((options & PARSE_PROLOG) != 0)
-      tr.setState(Compilation.PROLOG_PARSING);
-    ModuleExp module = tr.pushNewModule(lexer);
-    if ((options & PARSE_INTERACTIVE_MODULE) != 0)
-      {
-        tr.setInteractiveName();
-        module.setFlag(ModuleExp.INTERACTIVE);
-      }
-    if (info != null)
-      info.setCompilation(tr);
-    if (! parse(tr, options))
-      return null;
+      throws java.io.IOException, gnu.text.SyntaxException {
+      Compilation tr = getCompilation(lexer, options, info);
+      return parse(tr) ? tr : null;
+  }
+
+  public final boolean parse(Compilation tr)
+        throws java.io.IOException, gnu.text.SyntaxException {
+    try {
+        if (! parse(tr, tr.langOptions))
+            return false;
+    } catch (gnu.text.SyntaxException ex) {
+        tr.setState(Compilation.ERROR_SEEN);
+        return true;
+    }
     if (tr.getState() == Compilation.PROLOG_PARSING)
       tr.setState(Compilation.PROLOG_PARSED);
     else
-      module.classFor(tr);
-    return tr;
+      tr.mainLambda.classFor(tr);
+    return true;
   }
 
   public abstract boolean parse (Compilation comp, int options)
@@ -1127,9 +1138,13 @@ public abstract class Language
         boolean isStatic = (fld.getModifiers() & Access.STATIC) != 0;
         if (isAlias) {
             fdecl.setIndirectBinding(true);
-            if (ftype instanceof ClassType
-                && ((ClassType) ftype).isSubclass("gnu.mapping.DynamicLocation"))
-                fdecl.setFlag(Declaration.IS_DYNAMIC);
+            Type frtype = ftype.getRawType();
+            if (frtype instanceof ClassType) {
+                ClassType cftype = (ClassType) frtype;
+                if (cftype.isSubclass("gnu.mapping.DynamicLocation")
+                    || cftype.isSubclass("gnu.mapping.ThreadLocation"))
+                    fdecl.setFlag(Declaration.IS_DYNAMIC);
+            }
         }
         else if (isFinal && ftype instanceof ClassType) {
             if (ftype.isSubtype(Compilation.typeProcedure))
@@ -1195,18 +1210,8 @@ public abstract class Language
       Environment.setGlobal(Environment.getCurrent());
   }
 
-  public Procedure getPrompter()
-  {
-    Object property = null;
-    if (hasSeparateFunctionNamespace())
-      property = EnvironmentKey.FUNCTION;
-    Procedure prompter = (Procedure) getEnvironment()
-      .get(getSymbol("default-prompter"), property, null);
-    if (prompter != null)
-      return prompter;
-    else
-      return new SimplePrompter();
-  }
+    public String getPrimaryPrompt() { return "> "; }
+    public String getSecondaryPrompt() { return "- "; }
 
   /** Return the result of evaluating a string as a source expression. */
   public final Object eval (String string) throws Throwable
@@ -1288,8 +1293,9 @@ public abstract class Language
     Language saveLang = Language.setSaveCurrent(this);
     try
       {
-	Compilation comp = parse(port, messages, PARSE_FOR_EVAL);
+        Compilation comp = getCompilation(getLexer(port, messages), PARSE_FOR_EVAL, null);
         comp.setEvalName();
+        parse(comp);
 	ModuleExp.evalModule(getEnvironment(), ctx, comp, null, null);
       }
     finally
