@@ -44,8 +44,7 @@ public class SynchronizedExp extends Expression
     body.compileWithPosition(comp, target); 
     code.emitLoad(objvar);
     code.emitMonitorExit();
-    code.emitTryEnd();
-    code.emitCatchStart(null);
+    code.emitCatchStart((ClassType) null);
     code.emitLoad(objvar);
     code.emitMonitorExit(); 
     code.emitThrow(); 
