@@ -10,7 +10,8 @@ import gnu.lists.*;
 import gnu.commonlisp.lang.Symbols; // FIXME
 import gnu.text.Char;
 
-public abstract class Buffer extends AbstractSequence implements CharSeq
+public abstract class Buffer extends AbstractSequence<Char>
+    implements CharSeq
 {
   String name;
   Path path;
@@ -117,12 +118,10 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
     getStringContent().getChars(srcBegin, srcEnd, dst, dstBegin);
   }
 
-  /* #ifdef use:java.lang.CharSequence */
-  public CharSequence subSequence(int start, int end)
+  public CharSeq subSequence(int start, int end)
   {
     return getStringContent().subSequence(start, end);
   }
-  /* #endif */
   /* #ifdef JAVA5 */
   /**
    * @see gnu.lists.CharSeq#writeTo(int, int, Appendable)
