@@ -1,8 +1,11 @@
 package gnu.jemacs.buffer;
 import gnu.mapping.*;
 import gnu.expr.Language;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
+import gnu.kawa.io.TtyInPort;
 import java.io.*;
-import gnu.text.*;
 
 public class ReplMode extends ProcessMode
 {
