@@ -157,6 +157,10 @@ public class ApplyToArgs extends ProcedureN
                 return Char.valueOf(Strings.characterAt(str, iindex));
             }
         }
+        if (proc instanceof gnu.lists.Array) {
+            return ComposedArray.generalIndex((Array) proc, false,
+                                              1, args.length-1, args);
+        }
         if (proc instanceof List) {
             if (args.length != 2)
                 throw new WrongArguments(this, args.length); // FIXME
@@ -171,9 +175,6 @@ public class ApplyToArgs extends ProcedureN
 
             }
         }
-        if (proc instanceof gnu.lists.Array) {
-            return ArrayRef.arrayRef.applyN(args);
-        }
         /*
           What should happen if key has no associated value?
           Throw an exception?  Return null?
