@@ -46,7 +46,9 @@ public class CompileMisc
             args[1] = visitor.checkType(args[1], xtype);
             CompileReflect.checkKnownClass(type, comp);
             exp.setType(type);
-            if (args[1].getType() == type)
+            Type argType = args[1].getType();
+            if (argType != Type.nullType
+                && type.isCompatibleWithValue(argType) == 2)
                 return args[1];
             return exp;
           }
@@ -115,14 +117,29 @@ public class CompileMisc
   {
     exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
-    if (args.length == 1)
+    int nargs = args.length;
+    if (nargs == 1)
       return args[0];
-    if (args.length == 0)
-      return QuoteExp.voidExp;
+    if (nargs == 0)
+        return QuoteExp.voidExp;
     Expression folded = exp.inlineIfConstant(proc, visitor);
-    if (folded != exp)
-      return folded;
-    return exp;
+    if (folded.valueIfConstant() == Values.empty)
+        folded = QuoteExp.voidObjectExp;
+    else if (folded == exp) {
+        args = exp.getArgs();
+        Type typeSoFar = Type.voidType;
+        for (int i = 0;  i < nargs;  i++) {
+            Type atype = args[i].getType();
+            if (OccurrenceType.itemCountCode(atype) == '0')
+                continue;
+            if (typeSoFar == Type.voidType)
+                typeSoFar = atype;
+            else
+                typeSoFar = Type.objectType; // FIXME - can do better.
+        }
+        exp.setType(typeSoFar);
+    }
+    return folded;
   }
 
   public static Expression validateApplyMakeProcedure
@@ -194,11 +211,7 @@ public class CompileMisc
     // do a tree rewrite here, instead of in ValuesMap#compile.
     exp.visitArgs(visitor);
     if (lexp != null)
-      {
-	lexp.setInlineOnly(true);
-        lexp.returnContinuation = exp;
-        lexp.inlineHome = visitor.getCurrentLambda();
-      }
+        lexp.setInlineOnly(exp, visitor.getCurrentLambda());
     return exp;
   }
 
@@ -257,17 +270,16 @@ public class CompileMisc
         }
         CodeAttr code = comp.getCode();
         Type type = target.getType();
+        QuoteExp trueExp, falseExp;
         if (target instanceof StackTarget
                 && type.getSignature().charAt(0) == 'Z') {
-            arg.compile(comp, target);
-            code.emitNot(target.getType());
+            trueExp = QuoteExp.trueExp;
+            falseExp = QuoteExp.falseExp;
         } else {
-            QuoteExp trueExp
-                = QuoteExp.getInstance(language.booleanObject(true));
-            QuoteExp falseExp
-                = QuoteExp.getInstance(language.booleanObject(false));
-            IfExp.compile(arg, falseExp, trueExp, comp, target);
+            trueExp = QuoteExp.getInstance(language.booleanObject(true));
+            falseExp = QuoteExp.getInstance(language.booleanObject(false));
         }
+        IfExp.compile(arg, falseExp, trueExp, comp, target);
         return true;
     }
 
@@ -362,7 +374,13 @@ public class CompileMisc
                 if (kind0 <= Arithmetic.INTNUM_CODE
                     && kind1 <= Arithmetic.INTNUM_CODE
                     && (kind0 > Arithmetic.LONG_CODE
-                        || kind1 > Arithmetic.LONG_CODE)) {
+                        || kind1 > Arithmetic.LONG_CODE)
+                    && ! (kind0 == Arithmetic.ULONG_CODE
+                          && kind1 == Arithmetic.ULONG_CODE)) {
+                    // Use either IntNum.compare(intNum,IntNum)
+                    // or (if possible) IntNum.compare(IntNum,long).
+                    // Note we handle (ulong,int) and (ulong,long) by
+                    // delegating to IntNum.compare.  FIXME
                     Type[] ctypes = new Type[2];
                     ctypes[0] = Arithmetic.typeIntNum;
                     if (kind1 <= Arithmetic.LONG_CODE) {
@@ -383,7 +401,8 @@ public class CompileMisc
                         ctypes[1] = Arithmetic.typeIntNum;
                     Method cmeth
                         = Arithmetic.typeIntNum.getMethod("compare", ctypes);
-                    PrimProcedure compare = new PrimProcedure(cmeth);
+                    PrimProcedure compare =
+                        new PrimProcedure(cmeth, comp.getLanguage());
                     arg0 = new ApplyExp(compare, args);
                     arg1 = new QuoteExp(IntNum.zero());
                     kind0 = kind1 = Arithmetic.INT_CODE;
@@ -391,9 +410,37 @@ public class CompileMisc
                 Type commonType;
                 if (kind0 <= Arithmetic.INT_CODE && kind1 <= Arithmetic.INT_CODE)
                     commonType = Type.intType;
-                else if (kind0 <= Arithmetic.LONG_CODE && kind1 <= Arithmetic.LONG_CODE)
+                else if (kind0 <= Arithmetic.UINT_CODE
+                         && kind1 <= Arithmetic.UINT_CODE) {
+                    if (kind0 <= Arithmetic.INT_CODE
+                        || kind1 <= Arithmetic.INT_CODE) {
+                        // Mix of signed and unsigned int - use long
+                        commonType = Type.longType;
+                        kind0 = kind1 = Arithmetic.LONG_CODE;
+                    } else {
+                        // Both operands are unsigned int.
+                        Expression signBit =
+                            QuoteExp.makeShared(Integer.MIN_VALUE,
+                                                Type.intType);
+                        arg0 = new ApplyExp(AddOp.$Pl, arg0, signBit);
+                        arg1 = new ApplyExp(AddOp.$Pl, arg1, signBit);
+                        kind0 = kind1 = Arithmetic.INT_CODE;
+                        commonType = Type.intType;
+                    }
+                } else if (kind0 <= Arithmetic.LONG_CODE
+                           && kind1 <= Arithmetic.LONG_CODE) {
                     commonType = Type.longType;
-                else
+                } else if (kind0 <= Arithmetic.ULONG_CODE
+                           && kind1 <= Arithmetic.ULONG_CODE) {
+                    // Both operands unsigned long.
+                    // (Mix of unsigned long and signed int/long handled above.)
+                    Expression signBit =
+                        QuoteExp.makeShared(Long.MIN_VALUE, Type.longType);
+                    arg0 = new ApplyExp(AddOp.$Pl, arg0, signBit);
+                    arg1 = new ApplyExp(AddOp.$Pl, arg1, signBit);
+                    kind0 = kind1 = Arithmetic.LONG_CODE;
+                    commonType = Type.longType;
+                } else
                     commonType = Type.doubleType;
                 StackTarget subTarget = new StackTarget(commonType);
                 ConditionalTarget ctarget = (ConditionalTarget) target;
@@ -496,10 +543,10 @@ public class CompileMisc
             && (op == NumberPredicate.ODD || op == NumberPredicate.EVEN)) {
             Expression arg0 = args[0];
             int kind = Arithmetic.classifyType(arg0.getType());
+            CodeAttr code = comp.getCode();
             if (kind <= Arithmetic.INTNUM_CODE) {
                 PrimType wtype = Type.intType;
                 Target wtarget = StackTarget.getInstance(wtype);
-                CodeAttr code = comp.getCode();
                 if (op == NumberPredicate.EVEN)
                     code.emitPushInt(1);
                 arg0.compile(comp, wtarget);
@@ -507,9 +554,16 @@ public class CompileMisc
                 code.emitAnd();
                 if (op == NumberPredicate.EVEN)
                     code.emitSub(Type.intType);
-                target.compileFromStack(comp, Type.booleanType);
-                return true;
+            } else {
+                arg0.compile(comp, Target.pushObject);
+                String mname = op == NumberPredicate.EVEN ? "isEven" : "isOdd";
+                Method m = ClassType
+                    .make("gnu.kawa.functions.NumberPredicate")
+                    .getDeclaredMethod(mname, 1);
+                code.emitInvokeStatic(m);
             }
+            target.compileFromStack(comp, Type.booleanType);
+            return true;
         }
         return false;
     }
@@ -520,12 +574,11 @@ public class CompileMisc
     LambdaExp lexp = canInlineCallCC(exp);
     if (lexp != null)
       {
-	lexp.setInlineOnly(true);
-	lexp.returnContinuation = exp;
-        lexp.inlineHome = visitor.getCurrentLambda();
+        lexp.setInlineOnly(exp, visitor.getCurrentLambda());
         Declaration contDecl = lexp.firstDecl();
         if (! contDecl.getFlag(Declaration.TYPE_SPECIFIED))
           contDecl.setType(typeContinuation);
+        contDecl.setFlag(Declaration.DONT_COPY);
         LambdaExp.maybeSetReturnType(lexp, required);
         // FIXME (future): (after visiting lexp), do:
         // exp.setType(lexp.body.getType() UNION continuation-arguments);
@@ -581,10 +634,9 @@ public class CompileMisc
         code.emitPushInt(1);
         code.emitPutField(typeContinuation.getField("invoked"));
       }
-    code.emitTryEnd();
 
     // Emit: catch (Throwable ex) { handleException$(ex, cont, ctx); }
-    code.emitCatchStart(null);
+    code.emitCatchStart((ClassType) null);
     code.emitLoad(contVar);
     if (target instanceof ConsumerTarget)
       {
@@ -609,7 +661,8 @@ public class CompileMisc
   {
     Expression[] args = exp.getArgs();
     Expression arg0;
-    if (args.length == 1 && (arg0 = args[0]) instanceof LambdaExp)
+    if (args.length == 1 && (arg0 = args[0]) instanceof LambdaExp
+        && ! Compilation.enableANF)
       {
         LambdaExp lexp = (LambdaExp) arg0;
         if (lexp.min_args == 1 && lexp.max_args == 1
@@ -693,6 +746,25 @@ public class CompileMisc
         return exp;
     }
 
+    public static Expression validateApplyMakeDynamic
+        (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc) {
+        if (exp.isSimple(1, 1)) {
+            Expression[] args = exp.getArgs();
+            args[0] = visitor.visit(args[0], null);
+            exp.setType(LangObjType.dynamicType);
+        } else
+            exp.visitArgs(visitor);
+        return exp;
+    }
+
+    public static boolean compileMakeDynamic(ApplyExp exp, Compilation comp,
+                                             Target target,
+                                             Procedure procedure) {
+        if (! exp.isSimple())
+            return false;
+        exp.getArg(0).compile(comp, target);
+        return true;
+    }
 
   /** An ExpVisitor class to check if callcc exits through a try-finally. */
   static class ExitThroughFinallyChecker extends ExpVisitor<Expression,TryExp>
@@ -727,116 +799,6 @@ public class CompileMisc
     }
   }
 
-  public static Expression validateApplyMap
-  (ApplyExp exp, InlineCalls visitor, Type required, Procedure xproc)
-  {
-    Map mproc = (Map) xproc;
-    boolean collect = mproc.collect;
-    Expression[] args = exp.getArgs();
-    int nargs = args.length;
-    if (nargs < 2)
-      return exp;  // ERROR
-    
-    Type r = InlineCalls.typeForCalledFunction(args[0]);
-    if (r != null)
-      {
-        for (int i = 1; i < nargs;  i++)
-          visitor.visit(args[i], null);
-        // FIXME: We should inline the list arguments first before inlining the
-        // procedure argument, for better type inference etc.
-        visitor.visit(args[0], r);
-     }
-    else
-      exp.visitArgs(visitor);
-
-    nargs--;
-
-    Expression proc = args[0];
-    // If evaluating proc doesn't have side-effects, then we want to do
-    // so inside loop, since that turns a "read" info a "call", which
-    // may allow better inlining.
-    boolean procSafeForMultipleEvaluation = ! proc.side_effects();
-    Compilation comp = visitor.getCompilation();
-
-    // First an outer (let ((%proc PROC)) L2), where PROC is args[0].
-    if (! procSafeForMultipleEvaluation)
-      {
-        comp.letStart();
-        Declaration procDecl = comp.letVariable("%proc",
-                                                Compilation.typeProcedure,
-                                                proc);
-        proc = new ReferenceExp(procDecl);
-      }
-
-    // Then an inner L2=(let ((%loop (lambda (argi ...) ...))) (%loop ...))
-    comp.letStart();
-    LambdaExp lexp = new LambdaExp(collect ? nargs + 1 : nargs);
-    Declaration loopDecl = comp.letVariable("%loop", null, lexp);
-    comp.letEnter();
-
-    // Finally an inner L3=(let ((parg1 (as <pair> arg1)) ...) ...)
-    comp.letStart();
-    Declaration[] largs = new Declaration[nargs];
-    Declaration[] pargs = new Declaration[nargs];
-    for (int i = 0;  i < nargs;  i++)
-      {
-	String argName = "arg"+i;
-	largs[i] = lexp.addDeclaration(argName);
-	pargs[i] = comp.letVariable(argName, Compilation.typePair,
-                                    new ReferenceExp(largs[i]));
-      }
-    Declaration resultDecl = collect ? lexp.addDeclaration("result") : null;
-    Expression[] doArgs = new Expression[1+nargs];
-    Expression[] recArgs = new Expression[collect ? nargs + 1 : nargs];
-    for (int i = 0;  i < nargs;  i++)
-      {
-        doArgs[i+1] = SlotGet.makeGetField(new ReferenceExp(pargs[i]), "car");
-	recArgs[i] = SlotGet.makeGetField(new ReferenceExp(pargs[i]), "cdr");
-      }
-    doArgs[0] = proc;
-    Expression applyFunc = new ReferenceExp(SchemeCompilation.applyFieldDecl);
-    Expression doit = new ApplyExp(applyFunc, doArgs);
-    if (collect)
-      {
-	Expression[] consArgs = new Expression[2];
-	consArgs[0] = doit;
-	consArgs[1] = new ReferenceExp(resultDecl);
-	recArgs[nargs] = Invoke.makeInvokeStatic(Compilation.typePair,
-						 "make", consArgs);
-      }
-    Expression rec = new ApplyExp(new ReferenceExp(loopDecl), recArgs);
-    lexp.body = comp.letDone(collect ? rec : new BeginExp(doit, rec));
-    Expression[] initArgs = new Expression[collect ? nargs + 1 : nargs];
-    QuoteExp empty = new QuoteExp(LList.Empty);
-    for (int i = nargs;  --i >= 0; )
-      {
-	Expression[] compArgs = new Expression[2];
-	compArgs[0] = new ReferenceExp(largs[i]);
-	compArgs[1] = empty;
-	Expression result
-	  = collect ? (Expression) new ReferenceExp(resultDecl)
-	  : (Expression) QuoteExp.voidExp;
-	lexp.body = new IfExp(new ApplyExp(mproc.isEq, compArgs),
-			      result, lexp.body);
-	initArgs[i] = args[i+1];
-      }
-    if (collect)
-      initArgs[nargs] = empty;
-
-    Expression body = new ApplyExp(new ReferenceExp(loopDecl), initArgs);
-    if (collect)
-      {
-	Expression[] reverseArgs = new Expression[1];
-	reverseArgs[0] = body;
-	body = Invoke.makeInvokeStatic(Compilation.scmListType,
-				       "reverseInPlace", reverseArgs);
-      }
-    LetExp ret = comp.letDone(body);
-    if (! procSafeForMultipleEvaluation)
-      ret = comp.letDone(ret);
-    return visitor.visit(ret, null);
-  }
-
   public static Expression validateApplyMakePromise
   (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
