@@ -2,7 +2,8 @@ package gnu.kawa.io;
 import java.io.*;
 import gnu.mapping.Environment;
 import gnu.mapping.ThreadLocation;
-import gnu.text.*;
+import gnu.kawa.format.Printable;
+import gnu.kawa.format.AbstractFormat;
 import gnu.lists.*;
 
 /**
@@ -11,12 +12,18 @@ import gnu.lists.*;
 
 public class OutPort extends PrintConsumer implements Printable
 {
+    /** Output gets forwarded to the value of this field.
+     * It is initially a PrettyWriter, but other PrintConsumer object
+     * can be interjected for custom formatting or other processing. */
+    PrintConsumer formatter;
+
   Path path;
   protected Writer base;
   static final int FLUSH_ON_FINALIZE = 1;
   static final int CLOSE_ON_FINALIZE = 2;
   static final int IS_CLOSED = 4;
-  int finalizeAction;
+  static final int IS_DOMTERM = 8;
+    int flags;
 
   // To keep track of column-numbers, we use a helper class.
   // Otherwise, it is too painful, as there is no documented
@@ -30,8 +37,9 @@ public class OutPort extends PrintConsumer implements Printable
 
   protected OutPort(Writer base, PrettyWriter out, boolean autoflush)
   {
-    super(out, autoflush);
+    super((Consumer) out, autoflush);
     this.bout = out;
+    this.formatter = bout;
     this.base = base;
     if (closeOnExit())
       unregisterRef = WriterManager.instance.register(this);
@@ -92,15 +100,13 @@ public class OutPort extends PrintConsumer implements Printable
     this.path = path;
   }
 
-  public boolean printReadable;
-
     static BinaryOutPort outInitial
         = BinaryOutPort.makeStandardPort(System.out, "/dev/stdout");
-    static { outInitial.finalizeAction = FLUSH_ON_FINALIZE; }
+    static { outInitial.flags = FLUSH_ON_FINALIZE; }
 
     private static BinaryOutPort errInitial
         = BinaryOutPort.makeStandardPort(System.err, "/dev/stderr");
-    static { errInitial.finalizeAction = FLUSH_ON_FINALIZE; }
+    static { errInitial.flags = FLUSH_ON_FINALIZE; }
 
     public static BinaryOutPort getSystemOut() { return outInitial; }
     public static BinaryOutPort getSystemErr() { return errInitial; }
@@ -131,9 +137,7 @@ public class OutPort extends PrintConsumer implements Printable
     errLocation.set(e);
   }
 
-    public PrettyWriter getPrettyWriter() {
-        return bout;
-    }
+    public boolean isPrettyPrinting() { return bout.isPrettyPrinting(); }
 
     public static OutPort openFile(Object fname)
         throws java.io.IOException {
@@ -153,7 +157,7 @@ public class OutPort extends PrintConsumer implements Printable
                           ? new OutputStreamWriter(strm)
                           : new OutputStreamWriter(strm, conv.toString()),
                           path);
-        op.finalizeAction = CLOSE_ON_FINALIZE;
+        op.flags = CLOSE_ON_FINALIZE;
         return op;
     }
 
@@ -208,56 +212,34 @@ public class OutPort extends PrintConsumer implements Printable
     return Character.isJavaIdentifierPart(ch) || ch == '-' || ch == '+';
   }
 
-  //  java.text.FieldPosition fieldPosition;
-
-  /** If non-null, use this to print numbers. */
-  java.text.NumberFormat numberFormat;
-
-  public AbstractFormat objectFormat;
-
   @Override
   public void print(int v)
   {
-    if (numberFormat == null)
-      super.print(v);
-    else
-      print(numberFormat.format((long) v));
+    formatter.writeInt(v);
   }
 
   @Override
   public void print(long v)
   {
-    if (numberFormat == null)
-      super.print(v);
-    else
-      print(numberFormat.format(v));
+    formatter.writeLong(v);
   }
 
   @Override
   public void print(double v)
   {
-    if (numberFormat == null)
-      super.print(v);
-    else
-      print(numberFormat.format(v));
+    formatter.writeDouble(v);
   }
 
   @Override
   public void print(float v)
   {
-    if (numberFormat == null)
-      super.print(v);
-    else
-      print(numberFormat.format((double) v));
+    formatter.writeFloat(v);
   }
 
   @Override
   public void print(boolean v)
   {
-    if (objectFormat == null)
-      super.print(v);
-    else
-      objectFormat.writeBoolean(v, this);
+    formatter.writeBoolean(v);
   }
 
   @Override
@@ -269,12 +251,7 @@ public class OutPort extends PrintConsumer implements Printable
   @Override
   public void print(Object v)
   {
-    if (objectFormat != null)
-      objectFormat.writeObject(v, this);
-    else if (v instanceof Consumable)
-      ((Consumable) v).consume(this);
-    else
-      super.print(v == null ? "null" : v);
+    formatter.writeObject(v);
   }
 
   public void print (Consumer out)
@@ -288,25 +265,26 @@ public class OutPort extends PrintConsumer implements Printable
     out.write('>');
   }
 
-  @Override
+    @Override
+    public void startDocument() {
+        formatter.startDocument();
+    }
+
+    @Override
+    public void endDocument() {
+        formatter.endDocument();
+    }
+
+   @Override
   public void startElement (Object type)
   {
-    if (objectFormat != null)
-      objectFormat.startElement(type, this);
-    else
-      {
-	print('(');
-	print(type);
-      }
+    formatter.startElement(type);
   }
 
   @Override
   public void endElement ()
   {
-    if (objectFormat != null)
-      objectFormat.endElement(this);
-    else
-      print(')');
+    formatter.endElement();
   }
 
   /** Write a attribute for the current element.
@@ -314,30 +292,60 @@ public class OutPort extends PrintConsumer implements Printable
   @Override
   public void startAttribute (Object attrType)
   {
-    if (objectFormat != null)
-      objectFormat.startAttribute(attrType, this);
-    else
-      {
-        print(' ');
-        print(attrType);
-        print(": ");
-      }
+    formatter.startAttribute(attrType);
   }
 
   /** No more attributes in this element. */
   @Override
   public void endAttribute()
   {
-    if (objectFormat != null)
-      objectFormat.endAttribute(this);
-    else
-      print(' ');
+    formatter.endAttribute();
   }
 
-  /** Note the end of a "word".  See {@link #writeWordStart}. */
+    public void write(char[] str, int start, int count) {
+        formatter.write(str, start, count);
+    }
+
+    public void write(String str) {
+        formatter.write(str, 0, str.length());
+    }
+
+    public void write(String str, int start, int count) {
+        formatter.write(str, start, count);
+    }
+
+    public void write(CharSequence str, int start, int count) {
+        formatter.write(str, start, count);
+    }
+
+    public void writeObject(Object v) {
+        formatter.writeObject(v);
+    }
+
+    public void writeComment(char[] chars, int offset, int length) {
+        formatter.writeComment(chars, offset, length);
+    }
+
+    public void writeProcessingInstruction(String target, char[] content,
+                                           int offset, int length) {
+        formatter.writeProcessingInstruction(target, content, offset, length);
+    }
+
+    public void writeCDATA(char[] chars, int offset, int length) {
+        formatter.writeCDATA(chars, offset, length);
+    }
+
+    public void beginEntity (Object baseUri) {
+        formatter.beginEntity(baseUri);
+    }
+    public void endEntity() {
+        formatter.endEntity();
+    }
+
+    /** Note the end of a "word".  See {@link #writeWordStart}. */
   public void writeWordEnd ()
   {
-    bout.writeWordEnd();
+    formatter.writeWordEnd();
   }
 
   /** Maybe write a word-separating space.
@@ -346,13 +354,21 @@ public class OutPort extends PrintConsumer implements Printable
    */
   public void writeWordStart ()
   {
-    bout.writeWordStart();
+    formatter.writeWordStart();
   }
 
+    public Object pushFormat(AbstractFormat format) {
+        Object old = formatter;
+        formatter = format.makeConsumer(formatter);
+        return old;
+    }
+    public void popFormat(Object old) {
+        formatter = (PrintConsumer) old;
+    }
+
   public void freshLine()
   {
-    int col = bout.getColumnNumber();
-    if (col != 0)
+    if (! atLineStart())
       println();
   }
 
@@ -367,6 +383,10 @@ public class OutPort extends PrintConsumer implements Printable
     bout.setColumnNumber(column);
   }
 
+    public boolean atLineStart() {
+        return bout.atLineStart();
+    }
+
     void flushBuffer() {
         bout.forcePrettyOutput();
     }
@@ -395,7 +415,37 @@ public class OutPort extends PrintConsumer implements Printable
     unregisterRef = null;
   }
 
-    public boolean isOpen() { return (finalizeAction & IS_CLOSED) == 0; }
+    public boolean isOpen() { return (flags & IS_CLOSED) == 0; }
+    public boolean isDomTerm() { return (flags & IS_DOMTERM) != 0; }
+    public void setDomTerm(boolean v) {
+        if (v) flags |= IS_DOMTERM;
+        else flags &= ~IS_DOMTERM;
+        bout.isDomTerm = v;
+    }
+
+    /** Return an OutPort equivalent to the argumement for text output.
+     * I.e. writing CharSequences or characters to either is the same.
+     * Used to optimize process output.
+     */
+    public static OutPort getPassThroughOutPort(Consumer out) {
+        OutPort port = null;
+        for (;;) {
+            if (out instanceof OutPort) {
+                port = (OutPort) out;
+                PrintConsumer formatter = port.formatter;
+                if (formatter instanceof PrettyWriter)
+                    return port;
+                out = formatter;
+            } else if (out instanceof AbstractFormat.FormatConsumer) {
+                AbstractFormat.FormatConsumer fcons = (AbstractFormat.FormatConsumer) out;
+                if (!  fcons.getFormat().textIsCopied())
+                    return null;
+                out = fcons.getBase();
+            }
+            else
+                return port;
+        }
+    }
 
   @Override
   public void close()
@@ -417,7 +467,7 @@ public class OutPort extends PrintConsumer implements Printable
       }
     WriterManager.instance.unregister(unregisterRef);
     unregisterRef = null;
-    finalizeAction = IS_CLOSED;
+    flags = IS_CLOSED;
   }
 
   /** True if the port should be automatically closed on exit.
@@ -429,9 +479,9 @@ public class OutPort extends PrintConsumer implements Printable
 
   public void finalize ()
   {
-    if ((finalizeAction & FLUSH_ON_FINALIZE) != 0)
+    if ((flags & FLUSH_ON_FINALIZE) != 0)
       flush();
-    if ((finalizeAction & CLOSE_ON_FINALIZE) != 0)
+    if ((flags & CLOSE_ON_FINALIZE) != 0)
       close();
     else
       closeThis();
@@ -442,11 +492,14 @@ public class OutPort extends PrintConsumer implements Printable
     WriterManager.instance.run();
   }
 
-  public void startLogicalBlock (String prefix, boolean perLine,
-				 String suffix)
-  {
-    bout.startLogicalBlock(prefix, perLine, suffix);
-  }
+    public void setPrettyPrinting (boolean mode) {
+        bout.setPrettyPrinting(mode);
+    }
+
+    public void startLogicalBlock(String prefix, boolean perLine,
+                                  String suffix) {
+        bout.startLogicalBlock(prefix, perLine, suffix);
+    }
 
     public void startLogicalBlock(String prefix, String suffix,
                                   int indent) {
@@ -458,47 +511,24 @@ public class OutPort extends PrintConsumer implements Printable
         }
     }
 
-  public void endLogicalBlock (String suffix)
-  {
-    bout.endLogicalBlock(suffix);
-  }
-
-  public void writeBreak(int kind)
-  {
-    bout.writeBreak(kind);
-  }
-
-    public void writeSpaceLinear() {
-        synchronized(lock) {
-            write(' ');
-            writeBreak(PrettyWriter.NEWLINE_LINEAR);
-        }
+    public void endLogicalBlock(String suffix) {
+        bout.endLogicalBlock(suffix);
     }
 
-  /** Write a new-line iff the containing section cannot be printed
-   * on one line.  Either all linear-style newlines in a logical
-   * block becomes spaces (if it all fits in a line), or none
-   * of them do. */
-  public void writeBreakLinear()
-  {
-    writeBreak(PrettyWriter.NEWLINE_LINEAR);
-  }
+    public void writeBreak(int kind) {
+        bout.writeBreak(kind);
+    }
 
     /** Write a new-line if needed, space otherwise. */
-    public void writeSpaceFill() {
+    public void writeSpace(int kind) {
         synchronized (lock) {
             write(' ');
-            writeBreak(PrettyWriter.NEWLINE_FILL);
+            writeBreak(kind);
         }
     }
 
-  public void writeBreakFill()
-  {
-    writeBreak(PrettyWriter.NEWLINE_FILL);
-  }
-
-  public void setIndentation(int amount, boolean current)
-  {
-    bout.addIndentation(amount, current);
-  }
+    @Override
+    public void setIndentation(int amount, boolean current) {
+        bout.addIndentation(amount, current);
+    }
 }
