@@ -14,6 +14,7 @@ import gnu.kawa.reflect.CompileInvoke;
 import gnu.kawa.reflect.Invoke;
 import gnu.kawa.reflect.LazyType;
 import gnu.lists.Blob;
+import gnu.lists.FVector;
 import gnu.lists.Sequences;
 import gnu.lists.U8Vector;
 import java.util.*;
@@ -102,7 +103,7 @@ public class LangObjType extends SpecialObjectType implements TypeValue
                     VECTOR_TYPE_CODE);
 
   public static final LangObjType constVectorType =
-    new LangObjType("constant-vector", "gnu.lists.ConstVector",
+    new LangObjType("constant-vector", "gnu.lists.FVector",
                     CONST_VECTOR_TYPE_CODE);
 
   public static final LangObjType s8vectorType =
@@ -233,6 +234,8 @@ public class LangObjType extends SpecialObjectType implements TypeValue
 
     @Override
     public int isCompatibleWithValue(Type valueType) {
+        if (this == valueType)
+            return 2;
         switch (typeCode) {
         case SEQUENCE_TYPE_CODE:
             if (valueType instanceof ArrayType)
@@ -302,7 +305,6 @@ public class LangObjType extends SpecialObjectType implements TypeValue
       case U64VECTOR_TYPE_CODE:
       case F32VECTOR_TYPE_CODE:
       case F64VECTOR_TYPE_CODE:
-      case CONST_VECTOR_TYPE_CODE:
       case REGEX_TYPE_CODE:
         implementationType.emitIsInstance(comp.getCode());
         target.compileFromStack(comp,
@@ -375,7 +377,6 @@ public class LangObjType extends SpecialObjectType implements TypeValue
 
   public static Class coerceToClass (Object obj)
   {
-    obj = Promise.force(obj);
     Class coerced = coerceToClassOrNull(obj);
     if (coerced == null && obj != null)
       throw new ClassCastException("cannot cast "+obj+" to type");
@@ -426,6 +427,17 @@ public class LangObjType extends SpecialObjectType implements TypeValue
     return coerced;
   }
 
+    public static FVector coerceToConstVector(Object obj) {
+        obj = Promise.force(obj);
+        if (obj instanceof FVector) {
+            FVector vec = (FVector) obj;
+            if (vec.isReadOnly())
+                return vec;
+            throw new ClassCastException("vector is not constant-vector");
+        }
+       throw new ClassCastException("cannot cast "+obj+" to constant-vector");
+    }
+
   public static Procedure coerceToProcedureOrNull(Object value)
   {
     final Object obj = Promise.force(value);
@@ -491,8 +503,9 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         return typeLangObjType.getDeclaredMethod("coerceDFloNum", 1);
       case U8VECTOR_TYPE_CODE:
         return typeLangObjType.getDeclaredMethod("coerceToU8Vector", 1);
-      case VECTOR_TYPE_CODE:
       case CONST_VECTOR_TYPE_CODE:
+        return typeLangObjType.getDeclaredMethod("coerceToConstVector", 1);
+      case VECTOR_TYPE_CODE:
       case S8VECTOR_TYPE_CODE:
       case S16VECTOR_TYPE_CODE:
       case U16VECTOR_TYPE_CODE:
@@ -636,8 +649,9 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         return coerceDFloNum(obj);
       case SEQUENCE_TYPE_CODE:
           return Sequences.coerceToSequence(obj);
-      case VECTOR_TYPE_CODE:
       case CONST_VECTOR_TYPE_CODE:
+          return coerceToConstVector(obj);
+      case VECTOR_TYPE_CODE:
       case S8VECTOR_TYPE_CODE:
       case U8VECTOR_TYPE_CODE:
       case S16VECTOR_TYPE_CODE:
@@ -753,7 +767,6 @@ public class LangObjType extends SpecialObjectType implements TypeValue
   {
     switch (typeCode)
       {
-      case CONST_VECTOR_TYPE_CODE:
       case VECTOR_TYPE_CODE:
       case S8VECTOR_TYPE_CODE:
       case S16VECTOR_TYPE_CODE:
@@ -802,6 +815,8 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         return new PrimProcedure("gnu.kawa.io.URIPath", "makeURI", 1);
       case VECTOR_TYPE_CODE:
         return new PrimProcedure("gnu.lists.FVector", "make", 1);
+      case CONST_VECTOR_TYPE_CODE:
+          return new PrimProcedure("gnu.lists.FVector", "makeConstant", 1); 
       case LIST_TYPE_CODE:
         return gnu.kawa.functions.MakeList.list;
       case STRING_TYPE_CODE:
@@ -840,20 +855,20 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         switch (typeCode) {
         case S8VECTOR_TYPE_CODE:
         case U8VECTOR_TYPE_CODE:
-            return "byteAt";
+            return "getByte";
         case S16VECTOR_TYPE_CODE:
         case U16VECTOR_TYPE_CODE:
-            return "shortAt";
+            return "getShort";
         case S32VECTOR_TYPE_CODE:
         case U32VECTOR_TYPE_CODE:
-            return "intAt";
+            return "getInt";
         case S64VECTOR_TYPE_CODE:
         case U64VECTOR_TYPE_CODE:
-            return "longAt";
+            return "getLong";
         case F32VECTOR_TYPE_CODE:
-            return "floatAt";
+            return "getFloat";
         case F64VECTOR_TYPE_CODE:
-            return "doubleAt";
+            return "getDouble";
         }
         return null;
     }
@@ -862,8 +877,7 @@ public class LangObjType extends SpecialObjectType implements TypeValue
          String gname = elementGetterMethodName();
          if (gname == null)
              return null;
-         return "set" + Character.toUpperCase(gname.charAt(0))
-             + gname.substring(1);
+         return "set" + gname.substring(3);
     }
 
     public static final ClassType typeLangObjType =
