@@ -1,24 +1,44 @@
-(define-syntax process-keywords
-  (syntax-rules ()
-		((process-keywords obj args handle-keyword handle-non-keyword)
-		 (let ((num-args :: <int> (field args 'length)))
-		   (let loop ((i :: <int> 0))
-		     (if (< i num-args)
-			 (let ((arg ((primitive-array-get <object>) args i)))
-			   (cond ((instance? arg <gnu.expr.Keyword>)
-				  (handle-keyword obj
-						  (gnu.expr.Keyword:getName arg)
-						  ((primitive-array-get <object>) args (+ i 1)))
-				  (loop (+ i 2)))
-				 ((instance? arg <gnu.kawa.xml.KAttr>)
-				  (let* ((attr :: <gnu.kawa.xml.KAttr> arg)
-					 (name :: <java.lang.String> (invoke attr 'getName))
-					 (value (invoke attr 'getObjectValue)))
-				    (handle-keyword obj name value))
-				  (loop (+ i 1)))
-				 (else
-				  (handle-non-keyword obj arg)
-				  (loop (+ i 1)))))))))))
+(import (except (kawa pictures)
+                polygon)
+        (kawa process-keywords))
+
+(export make-Point circle polygon fill draw with-paint $construct$:P
+        button Button Image image-read image-width image-height color-red
+        with-composite as-color process-keywords
+        composite-src-over composite-src rotation with-transform Label Text Row Column set-content Window run-application)
+
+(define (polygon (initial :: <complex>) #!rest (more-points :: <object[]>))
+  (let ((path :: <java.awt.geom.GeneralPath>
+	      (make <java.awt.geom.GeneralPath>))
+	(n-points :: <int>
+		  ((primitive-array-length <object>) more-points)))
+    (path:moveTo ((real-part initial):doubleValue)
+		 ((imag-part initial):doubleValue))
+    (do ((i :: <int> 0 (+ i 1)))
+	((>= i n-points)
+	 (invoke path 'closePath)
+	 path)
+      (let ((pt :: <complex> ((primitive-array-get <object>) more-points i)))
+	(path:lineTo ((real-part pt):doubleValue)
+		     ((imag-part pt):doubleValue))))))
+
+(define (composite-src-over #!optional (alpha :: <float> 1.0))
+  :: <java.awt.Composite>
+  (java.awt.AlphaComposite:getInstance
+   (static-field <java.awt.AlphaComposite> 'SRC_OVER)
+   alpha))
+
+(define (composite-src #!optional (alpha :: <float> 1.0))
+  :: <java.awt.Composite>
+  (java.awt.AlphaComposite:getInstance
+   (static-field <java.awt.AlphaComposite> 'SRC)
+   alpha))
+
+(define (rotation (theta :: <double>)) :: <java.awt.geom.AffineTransform>
+  (java.awt.geom.AffineTransform:getRotateInstance theta))
+
+(define-constant color-red :: <java.awt.Color>
+  (static-field <java.awt.Color> 'red))
 
 (define (as-color value) :: <java.awt.Color>
   (cond ((instance? value <java.awt.Color>)
@@ -64,18 +84,9 @@
 
 (define-syntax Image
   (syntax-rules ()
-    ((text-field . args)
+    ((_ . args)
      (make <gnu.kawa.models.DrawImage> . args))))
 
-(define (image-read (uri :: path)) :: <java.awt.image.BufferedImage>
-  (javax.imageio.ImageIO:read (uri:openInputStream)))
-
-(define (image-width (image  :: <java.awt.image.BufferedImage>)) :: <int>
-  (*:getWidth image))
-
-(define (image-height (image  :: <java.awt.image.BufferedImage>)) :: <int>
-  (*:getHeight image))
-
 (define-private (label-keyword (instance :: <gnu.kawa.models.Label>)
 				(name :: <java.lang.String>)
 				value)
