@@ -19,7 +19,7 @@ public class GeneralHashTable<K,V>
     super(capacity);
   }
 
-  protected int getEntryHashCode (HashNode<K,V> entry) { return entry.hash; }
+  protected int getEntryHashCode (HashNode<K,V> entry) { return entry.keyHash; }
   protected HashNode<K,V> getEntryNext (HashNode<K,V> entry) { return entry.next; }
   protected void setEntryNext (HashNode<K,V> entry, HashNode<K,V> next) { entry.next = next; }
   protected HashNode<K,V>[] allocEntries(int n) { return (HashNode<K,V>[]) new HashNode[n]; }
@@ -27,9 +27,7 @@ public class GeneralHashTable<K,V>
   /** Allocate a new node in the hash table. */
   protected HashNode<K,V> makeEntry (K key, int hash, V value)
   {
-    HashNode<K,V> node = new HashNode<K,V>(key, value);
-    node.hash = hash;
-    return node;
+    return new HashNode<K,V>(key, value, hash);
   }
 
   /** This override helps Kawa type-inference - for example in srfi69.scm. */
