@@ -7,16 +7,12 @@ import gnu.kawa.util.HashUtils;
 import java.util.*;
 
 /**
- * An AbstractSequence is used to implement Sequences, and almost all
- * classes that extend AbstractSequence will implement Sequence.
- * However, AbstractSequence itself does not implement Sequence.
- * This is so we can use AbstractSequence to implement classes that are
- * "sequence-like" (such as multi-dimensional arrays) but are not Sequences.
+ * A collection of default methods for implementing sequence-like classes.
  *
  * Additionally, a sequence may have zero or more attributes, which are
  * name-value pairs.  A sequence may also have a named "type".  These
  * extensions are to support XML functionality - it might be cleaner to
- * moe them to a sub-class of Sequence or some interface.
+ * move them to a sub-class of Sequence or some interface.
  *
  * Many of the protected methods in Sequence (such as nextIndex) are
  * only intended to be called from SeqPosition or TreePosition, see those.
@@ -26,8 +22,19 @@ import java.util.*;
 
 public abstract class AbstractSequence<E>
 {
-  /** See java.util.List. */
-  public abstract int size();
+    public static final int[] noInts = new int[0];
+
+    public int size() {
+        if (rank() == 1) return getSize(0);
+        throw unsupported("size");
+    }
+
+    public int getSize() {
+        int sz = 1;
+        for (int r = rank(); --r >= 0; )
+            sz *= getSize(r);
+        return sz;
+    }
 
   public boolean isEmpty()
   {
@@ -39,23 +46,143 @@ public abstract class AbstractSequence<E>
     return 1;
   }
 
-  /** See java.util.List. */
-  public abstract E get (int index);
+    protected void checkRank(int i) {
+        if (i != rank())
+            throw badRank(i);
+    }
 
-  public int getEffectiveIndex(int[] indexes)
-  {
-    return indexes[0];
-  }
+    protected RuntimeException badRank(int i) {
+        return new RuntimeException("wrong number of indexes "+i+" to "+rank()+"-rank array");
+    }
 
-  public E get(int[] indexes)
-  {
-    return get(indexes[0]);
-  }
+    // FIXME remove this - some implementations needed
+    public Array<E> asImmutable() { throw unsupported("asImmutable"); }
 
-  public E set(int[] indexes, E value)
-  {
-    return set(indexes[0], value);
-  }
+    public E get() { return getRaw(effectiveIndex()); }
+    public E get(int i) {
+        return getRaw(effectiveIndex(i));
+    }
+    public E get(int i, int j) { return getRaw(effectiveIndex(i,j)); }
+    public E get(int i, int j, int k, int... rest) {
+        return getRaw(effectiveIndex(i, j, k, rest)); } 
+
+    public E get(int[] indexes) {
+        return getRaw(effectiveIndex(indexes));
+    }
+
+    protected void checkCanWrite () {
+    }
+    public E getRowMajor(int index) {
+        if (rank() == 1)
+            return get(index);
+        if (this instanceof Array)
+            return Arrays.getRowMajor((Array<E>) this, index);
+        throw unsupportedException("getRowMajor");
+    }
+
+    public int effectiveIndex() {
+        checkRank(0);
+        return 0;
+    }
+    public int effectiveIndex(int index) {
+        checkRank(1);
+        return index - getLowBound(0);
+    }
+    public int effectiveIndex(int i, int j) {
+        checkRank(2);
+        return (i - getLowBound(0)) * getSize(1) + (j - getLowBound(1));
+    }
+    public int effectiveIndex(int i, int j, int k, int... rest) {
+        int r = rest.length;
+        checkRank(3+r);
+        int eff = 0;
+        int stride = 1;
+        while (--r >= 0) {
+            eff += (rest[r] - getLowBound(3+r)) * stride;
+            stride *= getSize(3+r);
+        }
+        eff += (k - getLowBound(2)) * stride;
+        stride *= getSize(2);
+        eff += (j - getLowBound(1)) * stride;
+        stride *= getSize(1);
+        eff += (i - getLowBound(0)) * stride;
+        return eff;
+    }
+
+    public int effectiveIndex(int[] indexes) {
+        int ilength = indexes.length;
+        switch (indexes.length) {
+        case 0:
+            return effectiveIndex();
+        case 1:
+            return effectiveIndex(indexes[0]);
+        case 2:
+            return effectiveIndex(indexes[0], indexes[1]);
+        default:
+            int[] rest;
+            if (ilength == 3)
+                rest = noInts;
+            else {
+                rest = new int[ilength-3];
+                System.arraycopy(indexes, 3, rest, 0, ilength-3);
+            }
+            return effectiveIndex(indexes[0], indexes[1], indexes[2], rest);
+        }
+    }
+
+    public E getRaw(int index) { throw unsupported("getRaw"); }
+
+    protected void setBuffer(Object obj) { throw unsupported("setBuffer"); }
+
+    /** Given an "effective index", set selected element. */
+    public void setRaw(int index, E value) { throw unsupported("setRaw"); }
+
+    public boolean getBooleanRaw(int index) {
+        Object value = getRaw(index);
+        return value != null && ((Boolean) value).booleanValue();
+    }
+
+    public char getCharRaw(int index) {
+        return ((Character) getRaw(index)).charValue();
+    }
+
+    public byte getByteRaw(int index) {
+        return ((Number) getRaw(index)).byteValue();
+    }
+
+    public short getShortRaw(int index) {
+        return ((Number) getRaw(index)).shortValue();
+    }
+
+    public int getIntRaw(int index) {
+        return ((Number) getRaw(index)).intValue();
+    }
+
+    public long getLongRaw(int index) {
+        return ((Number) getRaw(index)).longValue();
+    }
+
+    public float getFloatRaw(int index) {
+        return ((Number) getRaw(index)).floatValue();
+    }
+
+    public double getDoubleRaw(int index) {
+        return ((Number) getRaw(index)).doubleValue();
+    }
+
+    public int getInt() { return getIntRaw(effectiveIndex()); }
+    public int getInt(int i) { return getIntRaw(effectiveIndex(i)); }
+    public int getInt(int i, int j) { return getIntRaw(effectiveIndex(i,j)); }
+    public int getInt(int i, int j, int k, int... rest) {
+        return getIntRaw(effectiveIndex(i, j, k, rest)); } 
+    public int getInt(int[] indexes) {
+        return getIntRaw(effectiveIndex(indexes));
+    }
+
+    public void set(int[] indexes, E value) {
+        checkCanWrite();
+        setRaw(effectiveIndex(indexes), value);
+    }
 
   public int getLowBound(int dim)
   {
@@ -67,7 +194,9 @@ public abstract class AbstractSequence<E>
     return dim==0 ? size() : 1;
   }
 
-  protected RuntimeException unsupported (String text)
+    public int getElementKind() { return Sequence.OBJECT_VALUE; }
+
+    protected RuntimeException unsupported (String text)
   {
     return unsupportedException(getClass().getName()
                                 + " does not implement " + text);
@@ -78,13 +207,12 @@ public abstract class AbstractSequence<E>
     return new UnsupportedOperationException(text);
   }
 
-  public E set(int index, E element)
-  {
-    throw unsupported("set");
-  }
-
-    public void setAt(int index, E element) {
-        throw unsupported("setAt");
+    public E set(int index, E value) {
+        checkCanWrite();
+        int effi = effectiveIndex(index);
+        E old = (E) getRaw(effi);
+        setRaw(effi, value);
+        return old;
     }
 
   public void fill(E value)
@@ -414,6 +542,9 @@ public abstract class AbstractSequence<E>
   protected int nextIndex(int ipos) {
     return ipos == -1 ? size() : ipos >>> 1;
   }
+    public int xnextIndex(int ipos) {
+        return nextIndex(ipos);
+    }
 
   protected int fromEndIndex(int ipos)
   {
@@ -680,6 +811,8 @@ public abstract class AbstractSequence<E>
 
   public int hashCode()
   {
+    if (rank() != 1 && this instanceof Array)
+        return Arrays.hashCode((Array) this);
     // Implementation specified by the Collections specification.
     int hash = 1;
     for (int i = startPos(); (i = nextPos(i)) != 0;  )
