@@ -375,6 +375,8 @@ public abstract class Environment
     Environment env = curEnvironment.get();
     if (env == null)
       {
+        if (Environment.global == null)
+          throw new Error("Environment.global not set - need to do Scheme.registerEnvironment() or similar");
 	env = Environment.make(Thread.currentThread().getName(), Environment.global);
         env.flags |= Environment.THREAD_SAFE;
         curEnvironment.set(env);
