@@ -6,6 +6,7 @@ import gnu.expr.*;
 import gnu.kawa.functions.DisplayFormat;
 import gnu.kawa.io.OutPort;
 import java.util.*;
+import gnu.kawa.lispexpr.LispLanguage;
 
 /** The translated form of a <code>(syntax <var>template</var>)</code>. */
 
@@ -79,7 +80,7 @@ public class SyntaxTemplate implements Externalizable {
 
     Object[] literal_values;
 
-    public static final SimpleSymbol dots3Symbol = Symbol.valueOf("...");
+    public static final SimpleSymbol dots3Symbol = LispLanguage.dots3_sym;
 
     /* DEBUGGING:
     void print_template_program(java.util.Vector patternNames,
@@ -202,14 +203,35 @@ public class SyntaxTemplate implements Externalizable {
                                  boolean isVector,
                                  Object ellipsis,
                                  Translator tr) {
+        Object unseeNeeded = null;
         if (form instanceof Pair || form instanceof FVector) {
             if (seen.containsKey(form)) {
-                /* FIXME cycles are OK if data are literal. */
-                tr.syntaxError("self-referential (cyclic) syntax template");
+                // FIXME cycles are OK if data are literal.
+                // Cycles in non-literal data could probably also be
+                // made to work with appropriate BUILD opcodes.
+                // However, that seems more trouble than it's worth.
+                tr.syntaxError("self-referential (cyclic) syntax template - "+form);
                 return -2;
             }
             seen.put(form, form);
+            unseeNeeded = form;
         }
+        int r = xconvert_template(form, syntax, template_program, nesting,
+                                  literals_vector, seen,
+                                  isVector, ellipsis, tr);
+        if (unseeNeeded != null)
+            seen.remove(unseeNeeded);
+        return r;
+    }
+    private int xconvert_template(Object form,
+                                 SyntaxForm syntax,
+                                 StringBuilder template_program,
+                                 int nesting,
+                                 java.util.Vector literals_vector,
+                                 IdentityHashMap seen,
+                                 boolean isVector,
+                                 Object ellipsis,
+                                 Translator tr) {
         while (form instanceof SyntaxForm) {
             syntax = (SyntaxForm) form;
             form = syntax.getDatum();
@@ -314,8 +336,11 @@ public class SyntaxTemplate implements Externalizable {
             // else treated quoted symbol as literal:
         }
         Object xform = tr.namespaceResolve(form);
-        if (xform instanceof Symbol)
+        Macro macro = tr.currentMacroDefinition;
+        if (xform instanceof Symbol && macro != null
+            && macro.capturedScope instanceof ModuleExp) {
             tr.noteAccess(xform, tr.currentScope());
+        }
         form = SyntaxForms.makeWithTemplate(syntax, form); // Usually a no-op.
         if (template_program.length() == 0
             && form instanceof PairWithPosition) {
