@@ -674,6 +674,26 @@ public class Translator extends Compilation
       pushForm(rewrite(exp, false));
   }
 
+    public int getCompletions(Environment env,
+                              String nameStart, Object property,
+                              String namespaceUri,
+                              List<? super String> matches) {
+        LocationEnumeration e = env.enumerateAllLocations();
+        int count = 0;
+        while (e.hasMoreElements()) {
+            Location loc = e.nextLocation();
+            Symbol sym = loc.getKeySymbol();
+            String local = sym == null ? null : sym.getLocalPart();
+            if (local != null && local.startsWith(nameStart)
+                && property == loc.getKeyProperty()
+                && namespaceUri == sym.getNamespaceURI()) {
+                count++;
+                matches.add(local);
+            }
+        }
+        return count;
+    }
+
     public Object namespaceResolve(Object name) {
         Object prefix = null;
         Expression part2 = null;
@@ -743,6 +763,26 @@ public class Translator extends Compilation
             return rewrite_pair((Pair) exp, function);
         else if (exp instanceof Symbol && ! selfEvaluatingSymbol(exp)) {
             Symbol s = (Symbol) exp;
+
+            // Check if we're handling a completion request.
+            int complete = s.getLocalName()
+                .indexOf(CommandCompleter.COMPLETE_REQUEST);
+            boolean separate = getLanguage().hasSeparateFunctionNamespace();
+            if (complete >= 0) {
+                List<String> candidates = new ArrayList<String>();
+                String prefix = s.toString().substring(0, complete);
+                Object property = function && separate ? EnvironmentKey.FUNCTION
+                    : null;
+                int symspace = function ? Language.FUNCTION_NAMESPACE
+                    : Language.VALUE_NAMESPACE;
+                getCompletions(env, prefix, property, s.getNamespaceURI(),
+                               candidates);
+                lexical.getCompletingSymbols(prefix, symspace,
+                                             candidates);
+                throw new CommandCompleter(complete, candidates,
+                                           prefix, prefix.length(), this);
+            }
+
             if (s.hasUnknownNamespace()) {
                 String loc = s.getLocalPart();
                 return rewrite_lookup(rewrite(Symbol.valueOf(s.getPrefix()), false),
@@ -821,7 +861,6 @@ public class Translator extends Compilation
                 nameToLookup = exp;
             }
             Symbol symbol = (Symbol) exp;
-            boolean separate = getLanguage().hasSeparateFunctionNamespace();
             if (decl != null) {
                 if (current_scope instanceof TemplateScope && decl.needsContext())
                     cdecl = ((TemplateScope) current_scope).macroContext;
@@ -1036,7 +1075,7 @@ public class Translator extends Compilation
             if (i < len && state > 1) {
                 DFloNum num = new DFloNum(name.substring(0, i));
                 boolean div = false;
-                Vector vec = new Vector();
+                ArrayList vec = new ArrayList();
                 for (; i < len;) {
                     char ch = name.charAt(i++);
                     if (ch == '*') {
@@ -1065,7 +1104,7 @@ public class Translator extends Compilation
                         }
                         ch = name.charAt(i++);
                     }
-                    vec.addElement(name.substring(unitStart, unitEnd));
+                    vec.add(name.substring(unitStart, unitEnd));
                     boolean expRequired = false;
                     if (ch == '^') {
                         expRequired = true;
@@ -1107,16 +1146,16 @@ public class Translator extends Compilation
                     }
                     if (neg)
                         exp = -exp;
-                    vec.addElement(IntNum.make(exp));
+                    vec.add(IntNum.make(exp));
                 }
                 if (i == len) {
                     int nunits = vec.size() >> 1;
                     Expression[] units = new Expression[nunits];
                     for (i = 0; i < nunits; i++) {
-                        String uname = (String) vec.elementAt(2 * i);
+                        String uname = (String) vec.get(2 * i);
                         Symbol usym = LispLanguage.unitNamespace.getSymbol(uname.intern());
                         Expression uref = tr.rewrite(usym);
-                        IntNum uexp = (IntNum) vec.elementAt(2 * i + 1);
+                        IntNum uexp = (IntNum) vec.get(2 * i + 1);
                         if (uexp.longValue() != 1)
                             uref = new ApplyExp(expt.expt,
                                                 new Expression[] {
@@ -1214,6 +1253,18 @@ public class Translator extends Compilation
             tr.error('w', "error loading class " + cname + " - " + ex.getMessage() + " not found");
         } catch (Exception ex) {
         }
+        if (name.startsWith("array")) {
+            int nlen = name.length();
+            if (nlen == 5)
+                return makeQuoteExp(GenArrayType.generalInstance);
+            try {
+                rank = Integer.parseInt(name.substring(5));
+                if (rank >= 0)
+                    return makeQuoteExp(new GenArrayType(rank, Type.objectType));
+            } catch (Throwable ex) {
+            }
+        }
+
         return null;
     }
 
@@ -1349,7 +1400,7 @@ public class Translator extends Compilation
 	     else
 	       error('e',
 		 "invalid type spec (must be \"type\" or 'type or <type>)");
-	     type = Type.pointer_type;
+             type = Type.errorType;
 	   }
         if (decl != null)
           decl.setType(texp, type);
@@ -1724,7 +1775,7 @@ public class Translator extends Compilation
   }
 
   /** Storage used by noteAccess and processAccesses. */
-  Vector notedAccess;
+  ArrayList notedAccess;
 
   /** Note that we reference name in a given scope.
    * This may be called when defining a macro, at scan-time,
@@ -1732,9 +1783,9 @@ public class Translator extends Compilation
   public void noteAccess (Object name, ScopeExp scope)
   {
     if (notedAccess == null)
-      notedAccess = new Vector();
-    notedAccess.addElement(name);
-    notedAccess.addElement(scope);
+      notedAccess = new ArrayList();
+    notedAccess.add(name);
+    notedAccess.add(scope);
   }
 
   /** Check references recorded by noteAccess.
@@ -1749,8 +1800,8 @@ public class Translator extends Compilation
     ScopeExp saveScope = current_scope;
     for (int i = 0;  i < sz;  i += 2)
       {
-	Object name = notedAccess.elementAt(i);
-	ScopeExp scope = (ScopeExp) notedAccess.elementAt(i+1);
+	Object name = notedAccess.get(i);
+	ScopeExp scope = (ScopeExp) notedAccess.get(i+1);
 	if (current_scope != scope)
           {
             // I.e. first time do equivalent of setPushCurrentScope
