@@ -282,39 +282,33 @@
   (gnu.kawa.functions.DisplayFormat:schemeWriteSimpleFormat:format
    value out))
 
-(define-private (%write-shared%
-                 fmt::gnu.kawa.functions.DisplayFormat
-                 value out::output-port) ::void
-  (let ((pretty-out (out:getPrettyWriter)))
-    (pretty-out:initialiseIDHash)
-    (pretty-out:setSharing #t)
-    (try-finally
-     (fmt:format value out)
-     (pretty-out:setSharing #f))
-    (pretty-out:finishIDHash)))
-
 (define (write
-	 value #!optional (out ::output-port (current-output-port))) ::void
-         (%write-shared%
-          (if (eqv? *print-circle* #t)
-              gnu.kawa.functions.DisplayFormat:schemeWriteSharedFormat
-              gnu.kawa.functions.DisplayFormat:schemeWriteFormat)
-          value out))
+	 value #!optional (out ::output-port (current-output-port)))
+  ::void
+  (*:format
+   (if (eqv? *print-circle* #t)
+       gnu.kawa.functions.DisplayFormat:schemeWriteSharedFormat
+       gnu.kawa.functions.DisplayFormat:schemeWriteFormat)
+   value out))
 
 (define (write-shared
-	 value #!optional (out ::output-port (current-output-port))) ::void
-         (%write-shared%
-          gnu.kawa.functions.DisplayFormat:schemeWriteSharedFormat
-          value out))
+	 value #!optional (out ::output-port (current-output-port)))
+  ::void
+  (*:format
+   gnu.kawa.functions.DisplayFormat:schemeWriteSharedFormat
+   value out))
 
 (define (write-with-shared-structure 
-	 value #!optional (out ::output-port (current-output-port))) ::void
-         (%write-shared%
-          gnu.kawa.functions.DisplayFormat:schemeWriteSharedFormat
-          value out))
+	 value #!optional (out ::output-port (current-output-port)))
+  ::void
+  (*:format
+   gnu.kawa.functions.DisplayFormat:schemeWriteSharedFormat
+   value out))
     
 (define (display value #!optional (out (current-output-port))) :: <void>
-  (*:format gnu.kawa.functions.DisplayFormat:schemeDisplayFormat value out))
+  (*:format
+   gnu.kawa.functions.DisplayFormat:schemeDisplayFormat
+   value out))
 
 (define (input-port-read-state port)
   ((primitive-virtual-method <input-port> "getReadState" <char> ())
@@ -346,16 +340,6 @@
 (define (input-port-column-number port)
   (+ 1 (port-column port)))
 
-(define (default-prompter port)
-  (let ((state (input-port-read-state port)))
-    (if (char=? state #\Newline)
-	""
-	(string-append (if (char=? state #\Space)
-			   "#|kawa:"
-			   (string-append "#|" (make-string 1 state) "---:"))
-		       (number->string (input-port-line-number port))
-		       "|# "))))
-
 (define (set-input-port-prompter!
 	 (port :: <gnu.kawa.io.TtyInPort>) (prompter :: <procedure>))
   (invoke port 'setPrompter prompter))
