@@ -1,4 +1,4 @@
 (module-name (scheme lazy))
 (require kawa.lib.misc)
 (require kawa.lib.std_syntax)
-(export delay delay-force force make-promise promise?)
+(include "lazy-exports")
