@@ -278,11 +278,19 @@
 
 (define-syntax $bracket-list$
   (syntax-rules (<: <=: >: >=:)
-    ((_ x <: y) (invoke-static gnu.lists.Range 'valueOfLT x y))
-    ((_ x <=: y) (invoke-static gnu.lists.Range 'valueOfLE x y))
-    ((_ x >: y) (invoke-static gnu.lists.Range 'valueOfGT x y))
-    ((_ x >=: y) (invoke-static gnu.lists.Range 'valueOfGE x y))
-    ((_ x <:) (invoke-static gnu.lists.Range 'valueOfUnbounded x))
+    ((_ x <: y) (gnu.kawa.functions.RangeUtils:upto x 1 y #f))
+    ((_ x <=: y) (gnu.kawa.functions.RangeUtils:upto x 1 y #t))
+    ((_ x >: y) (gnu.kawa.functions.RangeUtils:downto x -1 y #f))
+    ((_ x >=: y) (gnu.kawa.functions.RangeUtils:downto x -1 y #t))
+    ((_ x by: step <: y) (gnu.kawa.functions.RangeUtils:upto x step y #f))
+    ((_ x by: step <=: y) (gnu.kawa.functions.RangeUtils:upto x step y #t))
+    ((_ x by: step >: y) (gnu.kawa.functions.RangeUtils:downto x step y #f))
+    ((_ x by: step >=: y) (gnu.kawa.functions.RangeUtils:downto x step y #t))
+    ((_ x <:) (gnu.kawa.functions.RangeUtils:valueOfUnbounded x 1))
+    ((_ x by: step) (gnu.kawa.functions.RangeUtils:valueOfUnbounded x step))
+    ((_ x by: step size: sz) (gnu.kawa.functions.RangeUtils:bySize x step sz))
+    ((_ <:) (gnu.lists.Range$IntRange 0 1 -2))
+    ((_ >:) (gnu.lists.Range$IntRange 0 -1 -2))
     ((_ . args) (constant-vector . args))))
 
 (cond-expand
@@ -309,7 +317,7 @@
        (let ((cname (datum->syntax #'name (%symbol->construct (syntax->datum #'name)))))
          #`(define-syntax #,cname
              (syntax-rules ()
-               ((#,cname . args)
+               ((_ . args)
                 (%simple-construct-builder constructor text-collector () . args))))))
       ((_ name constructor)
        #`(define-simple-constructor name constructor $string$)))))
@@ -318,6 +326,8 @@
   (syntax-rules ($<<$ $>>$)
     ((%simple-construct-builder fun mkstr (seen ...) $<<$ . rest)
      (fun (mkstr seen ... $<<$ . rest)))
+    ((%simple-construct-builder fun mkstr (seen ...) $>>$)
+     (fun seen ...))
     ((%simple-construct-builder fun mkstr (seen ...) $>>$ . rest)
      (fun seen ... (mkstr . rest)))
     ((%simple-construct-builder fun mkstr (seen ...) x . rest)
