@@ -7,7 +7,7 @@ import gnu.expr.*;
 import java.util.*;
 import gnu.text.SourceMessages;
 import gnu.kawa.lispexpr.*;
-import gnu.lists.AbstractFormat;
+import gnu.kawa.format.AbstractFormat;
 import gnu.kawa.functions.*;
 import gnu.kawa.io.CharArrayInPort;
 import gnu.kawa.io.InPort;
@@ -512,10 +512,13 @@ public class Scheme extends LispLanguage {
       defProcStFld("set-input-port-line-number!", "kawa.lib.ports");
       defProcStFld("input-port-column-number", "kawa.lib.ports");
       defProcStFld("input-port-read-state", "kawa.lib.ports");
-      defProcStFld("default-prompter", "kawa.lib.ports");
+      defProcStFld("default-prompter", "kawa.Shell", "defaultPrompter");
       defProcStFld("input-port-prompter", "kawa.lib.ports");
       defProcStFld("set-input-port-prompter!", "kawa.lib.ports");
+      defAliasStFld("input-prompt1", "gnu.kawa.io.CheckConsole", "prompt1");
+      defAliasStFld("input-prompt2", "gnu.kawa.io.CheckConsole", "prompt2");
       defProcStFld("base-uri", "kawa.lib.misc");
+      defProcStFld("domterm-load-stylesheet", "kawa.lib.kawa.domterm");
 
       defProcStFld("syntax-error", "kawa.standard.syntax_error",
                    "syntax_error");
@@ -850,14 +853,30 @@ public class Scheme extends LispLanguage {
 
       defProcStFld("array?", "kawa.lib.arrays");
       defProcStFld("array-rank", "kawa.lib.arrays");
+      defProcStFld("array-size", "kawa.lib.arrays");
       defProcStFld("make-array", "kawa.lib.arrays");
-      defProcStFld("array", "kawa.lib.arrays");
+      defAliasStFld("array",
+                    "gnu.kawa.lispexpr.GenArrayType", "generalInstance");
       defProcStFld("array-start", "kawa.lib.arrays");
       defProcStFld("array-end", "kawa.lib.arrays");
       defProcStFld("shape", "kawa.lib.arrays");
       defProcStFld("array-ref", "gnu.kawa.functions.ArrayRef", "arrayRef");
       defProcStFld("array-set!", "gnu.kawa.functions.ArraySet", "arraySet");
+      defProcStFld("array-fill!", "kawa.lib.arrays");
+      defProcStFld("array-copy!", "kawa.lib.arrays");
       defProcStFld("share-array", "kawa.lib.arrays");
+      defProcStFld("array-index-ref", "kawa.lib.arrays");
+      defProcStFld("array-index-share", "kawa.lib.arrays");
+      defProcStFld("array-transform", "kawa.lib.arrays");
+      defProcStFld("array-reshape", "kawa.lib.arrays");
+      defProcStFld("array-flatten", "kawa.lib.arrays");
+      defProcStFld("array->vector", "kawa.lib.arrays");
+      defProcStFld("index-array", "kawa.lib.arrays");
+      defProcStFld("build-array", "kawa.lib.arrays");
+      // array->mutable-array arr -- or plain array-copy
+      // array-map
+      // list*->array and array->list*
+      defProcStFld("format-array", "kawa.lib.arrays");
 
       for (int i = uniformVectorTags.length;  --i >= 0; )
         {
@@ -1117,6 +1136,20 @@ public class Scheme extends LispLanguage {
             sbuf.append(']');
             return sbuf.toString();
         }
+        if (type instanceof GenArrayType) {
+            GenArrayType atype = (GenArrayType) type;
+            StringBuilder sbuf = new StringBuilder("array");
+            int rank = atype.rank();
+            if (rank >= 0)
+                sbuf.append(rank);
+            Type elementType = atype.getComponentType();
+            if (elementType != null && elementType != Type.objectType) {
+                sbuf.append('[');
+                sbuf.append(formatType(elementType));
+                sbuf.append(']');
+            }
+            return sbuf.toString();
+        }
         if (typeToStringMap == null)  {
             typeToStringMap = new HashMap<Type,String>();
             // Invert the map returned by getTypeMap.
