@@ -18,12 +18,8 @@ public class begin extends Syntax
     return tr.rewrite_body (obj);
   }
 
-  public void scanForm (Pair st, ScopeExp defs, Translator tr)
-  {
-    Object body = tr.scanBody(st.getCdr(), defs, true);
-    if (body != LList.Empty)
-      // Because rewrite to be called later, with whatever is left after
-      // removing declarations.
-      tr.formStack.add(Translator.makePair(st, st.getCar(), body));
-  }
+    @Override
+    public void scanForm(Pair st, ScopeExp defs, Translator tr) {
+        tr.scanBody(st.getCdr(), defs, false);
+    }
 }
