@@ -2,13 +2,14 @@ package kawa.standard;
 import java.io.*;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.io.Path;
 import gnu.text.SourceMessages;
 import gnu.text.SyntaxException;
 import gnu.lists.*;
 import gnu.text.*;
 import kawa.Shell;
 
-public class load extends Procedure1 {
+public class load extends Procedure1or2 {
   boolean relative;
 
   public load (String name, boolean relative)
@@ -51,7 +52,7 @@ public class load extends Procedure1 {
               { 
                 clas = Class.forName(name.toString());
               }
-            catch (Throwable e2)
+            catch (Exception e2)
               {
                 // Handled below since clas==null
               }
