@@ -79,9 +79,12 @@ public class require extends Syntax
         map("list-lib", SLIB_PREFIX + "srfi1");
         map("srfi-2", SLIB_PREFIX + "srfi2");
         map("and-let*", SLIB_PREFIX + "srfi2");
+        map("srfi-8", SLIB_PREFIX + "receive");
+        map("receive", SLIB_PREFIX + "receive");
         map("srfi-13", SLIB_PREFIX + "srfi13");
         map("srfi-14", SLIB_PREFIX + "srfi14");
         map("string-lib", SLIB_PREFIX + "srfi13");
+        map("srfi-26", SLIB_PREFIX + "cut");
         map("srfi-34", SLIB_PREFIX + "srfi34");
         map("srfi-35", SLIB_PREFIX + "conditions");
         map("condition", SLIB_PREFIX + "conditions");
@@ -167,6 +170,7 @@ public class require extends Syntax
                 && requestedClass.charAt(nlen-1) == '>')
                 requestedClass = requestedClass.substring(1, nlen-1);
             String implicitSource = requestedClass.replace('.', '/');
+            requestedClass = Compilation.mangleQualifiedName(requestedClass);
             String explicitSource = null;
             if (args.getCdr() instanceof Pair) {
                 Object sname = ((Pair) args.getCdr()).getCar();
@@ -202,7 +206,10 @@ public class require extends Syntax
     public static ModuleInfo lookupModuleFromSourcePath (String sourceName, ScopeExp defs) {
         ModuleManager manager = ModuleManager.getInstance();
         String baseName = defs.getFileName();
-        if (baseName != null && baseName != InPort.systemInFilename)
+        if (baseName != null
+            && baseName != InPort.systemInFilename
+            && baseName != InPort.stringPathname
+            && baseName != InPort.evalPathname)
             sourceName = Path.valueOf(baseName).resolve(sourceName).toString();
         return manager.findWithSourcePath(sourceName);
     }
@@ -216,10 +223,9 @@ public class require extends Syntax
                       DeclSetMapper mapper, FormStack forms, 
                       ScopeExp defs, Compilation tr) {
         ModuleManager manager = ModuleManager.getInstance();
-        long now;
         if ((info.getState() & 1) == 0
             && info.getCompilation() == null
-            && ! info.checkCurrent(manager, (now = System.currentTimeMillis()))) {
+            && ! info.checkCurrent(manager, System.currentTimeMillis())) {
             SourceMessages messages = tr.getMessages();
             Language language = Language.getDefaultLanguage();
             Compilation comp;
@@ -230,6 +236,8 @@ public class require extends Syntax
                 if (tr.immediate)
                     options |= Language.PARSE_IMMEDIATE;
                 comp = language.parse(fstream, messages, options, info);
+                if (tr.getModule().getFlag(ModuleExp.INTERACTIVE));
+                    comp.getModule().setFlag(ModuleExp.INTERACTIVE);
             } catch (java.io.FileNotFoundException ex) {
                 tr.error('e', "not found: "+ex.getMessage());
                 return false;
@@ -300,8 +308,8 @@ public class require extends Syntax
 
         ModuleExp mod = info.setupModuleExp();
 
-        Map<Symbol,Declaration> dmap
-            = new LinkedHashMap<Symbol,Declaration>();
+        Map<Symbol,Expression> dmap
+            = new LinkedHashMap<Symbol,Expression>();
         Map<String,Declaration> moduleReferences = null;
        
         for (Declaration fdecl = mod.firstDecl();
@@ -324,15 +332,17 @@ public class require extends Syntax
                     moduleReferences = new HashMap<String,Declaration>();
                 moduleReferences.put(fdecl.field.getName(), fdecl);
             } else
-                dmap.put((Symbol) fdecl.getSymbol(), fdecl);
+                dmap.put((Symbol) fdecl.getSymbol(),
+                         new ReferenceExp(fdecl));
         }
 
         if (mapper != null)
             dmap = mapper.map(dmap, tr);
 
-        for (Map.Entry<Symbol,Declaration> entry : dmap.entrySet()) {
+        for (Map.Entry<Symbol,Expression> entry : dmap.entrySet()) {
             Symbol aname = entry.getKey();
-            Declaration fdecl = entry.getValue();
+            ReferenceExp fref = (ReferenceExp) entry.getValue();
+            Declaration fdecl = fref.getBinding();
 
             // We create an alias in the current context that points
             // a dummy declaration in the exported module.  Normally,
@@ -384,7 +394,6 @@ public class require extends Syntax
             adecl.setAlias(true);
             adecl.setIndirectBinding(true);
 
-            ReferenceExp fref = new ReferenceExp(fdecl);
             fref.setContextDecl(decl);
             fref.setDontDereference(true);
             if (! sharedModule)
@@ -468,6 +477,6 @@ public class require extends Syntax
     }
 
     public static interface DeclSetMapper {
-        public Map<Symbol, Declaration> map(Map<Symbol, Declaration> decls, Compilation comp);
+        public Map<Symbol, Expression> map(Map<Symbol, Expression> decls, Compilation comp);
     }
 }
